/**
 * Controller for style editor template for plot widget
 *
 */
mod.controller(
    'plotStylerController',
    [
        '$scope',
        'plugin-plotWidget.factories.editorStyleFactory',
        'basicchart.services.$highchart',
        'plugin-plotWidget.services.$plotHighchart',
        'basicchart.services.$highchart.stylerUpdate',
        function ($scope, editorStyleFactory, $highchart, $plotHighchart, $stylerUpdate) {

            /**
             * private methods
             */
            (function init() {
                setDefaultAxisTitles();
            })();

            function setDefaultAxisTitles() {
                var defaultAxisTitles = $plotHighchart.getAxisTitles($scope.widget, 'plotwidget');
                $scope.defaultXAxisTitle = defaultAxisTitles.x1;
                $scope.defaultYAxisTitle = defaultAxisTitles.y1;
            }


            // array for checkboxes
            $scope.whiskerOptions = editorStyleFactory.WHISKER_OPTIONS_array;
            $scope.typesArray = editorStyleFactory.BOXPLOT_TYPE_array;


            $scope.$watch('widget.queryResult', function(nv){
                    //fix. one time calls empty model
                    defined(nv) && defined($scope.model) && $stylerUpdate.enableIntervals($scope.model, $scope.widget.queryResult);
             });

            /**
             * watches
             */
            // updates widget's styles object
            var unbindWidgetWatcher = $scope.$watch('widget', function (val) {
                $scope.model = $$get($scope, 'widget.style');
                $stylerUpdate.enableIntervals($scope.model, $scope.widget.queryResult);

            });

            /**
             * public methods
             */

            $scope.$watch('widget.subtype', function (type) {

                if (!defined(type)) {
                    return;
                }

                $stylerUpdate.enableIntervals($scope.model, $scope.widget.queryResult);
            });

                // handler for click on Tooltip radio buttons on styles panel
            $scope.clickTipRadioBox = function (propertyName) {
                if (defined($scope.model.whisker[propertyName])) {
                    $scope.model.whisker['whisker/deviation'] = $scope.model.whisker['whisker/extremums'] = $scope.model.whisker['whisker/iqr'] = false;
                    $scope.model.whisker[propertyName] = true;
                    $scope.refresh();
                }
            };


            /**
             * public methods
             */
            $scope.changeType = function (type) {

                // validation
                if (!defined(type, "value") || type.value === $scope.widget.subtype) {
                    return;
                }

                $scope.widget.subtype = type.value;

                //redrawChart do we need redrawCallFromController ?
                $scope.widget.redraw();
            };

            $scope.refresh = function () {
                _.defer(function () {
                    $scope.$root.widget.refresh();
                });
            };

            // used in style editor template checkboxes and radio buttons
            $scope.isSelectedWhisker = function (propertyName) {

                try {
                    return $$get($scope, "model.whisker." + propertyName);

                } catch (ex) {
                    //debugger;
                }

            };

            $scope.redrawChart = function () {

                $highchart.redrawCallFromController($scope);
            };


            // redraws widget
            $scope.redraw = function () {
                _.defer(function () {
                    if (defined($scope.$root.widget, 'rawQueryResult'))
                        $scope.$root.widget.redraw();
                });
            };

            // destroying widget and unbinding watchers
            var destroyUnregisterer = $scope.$on("$destroy", function () {
                unbindWidgetWatcher();
                destroyUnregisterer();
            });

        }

    ]);
