prism.run([

    // using $df to register basic chart widgets
    function () {

        // Function first version
        //triggers original function, deletes errors and enables labels and gridLines
        (function (Highcharts) {

            var each = Highcharts.each,
                UNDEFINED = undefined,
                NORMAL_STATE = '',
                HIDDEN = 'hidden',
                VISIBLE = 'visible',
                math = Math,
                mathRound = math.round,
                mathFloor = math.floor,
                mathMax = math.max,
                merge = Highcharts.merge,
                extend = Highcharts.extend,
                pick = Highcharts.pick;

            //changes default Highcharts functions
            Highcharts.getOptions().plotOptions.boxplot.states.select["strokeOpacity"] = 0.25;
            Highcharts.getOptions().plotOptions.boxplot.states.select["fillOpacity"] = 0.25;
            Highcharts.getOptions().plotOptions.boxplot.states.hover["lineWidth"] = 2;


            Highcharts.wrap(Highcharts.seriesTypes.boxplot.prototype, 'drawPoints', function (defaultCallback) {
                var series = this,  //state = series.state,
                    points = series.points,
                    options = series.options,
                    chart = series.chart,
                    renderer = chart.renderer,
                    pointAttr,
                    pointAttrNew,
                    innerPointAttr,
                    q1Plot,
                    q3Plot,
                    highPlot,
                    lowPlot,
                    medianPlot,
                    crispCorr,
                    crispX,
                    graphic,
                    stemPath,
                    stemAttr,
                    boxPath,
                    whiskersPath,
                    innerBoxPath,
                    whiskersAttr,
                    medianPath,
                    medianAttr,
                    width,
                    left,
                    right,
                    halfWidth,
                    shapeArgs,
                    color,
                    innerBoxColor,
                    doQuartiles = series.doQuartiles !== false, // error bar inherits this series type but doesn't do quartiles
                    whiskerLength = parseInt(series.options.whiskerLength, 10) / 100;

                /* Sisense extension apply maxPointWidth */
                if (series.data.length > 0) {
                    var width = series.barW;

                    if (series.barW > series.options.maxPointWidth) {
                        width = series.options.maxPointWidth;
                    }
                    if (series.barW < series.options.minPointWidth) {
                        width = series.options.minPointWidth;
                    }

                    each(this.data, function (point) {
                        point.shapeArgs.x += (point.shapeArgs.width - width) / 2;
                        point.shapeArgs.width = width;
                    });
                }
                /*Sisense extension end */

                each(points, function (point) {

                    graphic = point.graphic;
                    shapeArgs = point.shapeArgs; // the box
                    stemAttr = {};
                    whiskersAttr = {};
                    medianAttr = {};
                    color = point.color || series.color;
                    innerBoxColor = point.innerBoxColor || series.color;


                    if (point.plotY !== UNDEFINED) {

                        pointAttr = point.pointAttr[point.selected ? 'select' : NORMAL_STATE] || point.pointAttr[NORMAL_STATE];

                        //Sisense fix start
                        point.selected && ( pointAttr['fill'] = point.pointAttr[NORMAL_STATE].fill);
                        innerPointAttr = {
                            'fill': innerBoxColor,
                            'stroke': 'transparent',
                            'stroke-width': '1'
                        };

                        var graphicAttr = {
                            'fill': pointAttr['fill'],
                            'fill-opacity': pointAttr['fill-opacity'],
                            'stroke-opacity': pointAttr['fill-opacity']
                        };
                        pointAttrNew = $.extend({}, {}, pointAttr);
                        delete pointAttrNew['fill-opacity'];
                        delete pointAttrNew['stroke-opacity'];
                        //pointAttrNew['fill'] = "wrong-color";

                        //Sisense extension end

                        // crisp vector coordinates
                        width = shapeArgs.width;
                        left = mathFloor(shapeArgs.x);
                        right = left + width;
                        halfWidth = mathRound(width / 2);
                        //crispX = mathRound(left + halfWidth) + crispCorr;
                        q1Plot = mathFloor(doQuartiles ? point.q1Plot : point.lowPlot);// + crispCorr;
                        q3Plot = mathFloor(doQuartiles ? point.q3Plot : point.lowPlot);// + crispCorr;
                        highPlot = mathFloor(point.highPlot);// + crispCorr;
                        lowPlot = mathFloor(point.lowPlot);// + crispCorr;

                        // Stem attributes
                        stemAttr.stroke = point.stemColor || options.stemColor || color;
                        stemAttr['stroke-width'] = pick(point.stemWidth, options.stemWidth, options.lineWidth);
                        stemAttr.dashstyle = point.stemDashStyle || options.stemDashStyle;

                        // Whiskers attributes
                        whiskersAttr.stroke = point.whiskerColor || options.whiskerColor || color;
                        whiskersAttr['stroke-width'] = pick(point.whiskerWidth, options.whiskerWidth, options.lineWidth);


                        // Median attributes
                        medianAttr.stroke = point.medianColor || options.medianColor || color;
                        medianAttr['stroke-width'] = pick(point.medianWidth, options.medianWidth, options.lineWidth);
                        medianAttr['stroke-linecap'] = 'round'; // #1638


                        // The stem
                        crispCorr = (stemAttr['stroke-width'] % 2) / 2;
                        crispX = left + halfWidth + crispCorr;
                        stemPath = [
                            // stem up
                            'M',
                            crispX, q3Plot,
                            'L',
                            crispX, highPlot,

                            // stem down
                            'M',
                            crispX, q1Plot,
                            'L',
                            crispX, lowPlot,
                            'z'
                        ];

                        // The box
                        if (doQuartiles) {
                            crispCorr = (pointAttr['stroke-width'] % 2) / 2;
                            crispX = mathFloor(crispX) + crispCorr;
                            q1Plot = mathFloor(q1Plot) + crispCorr;
                            q3Plot = mathFloor(q3Plot) + crispCorr;
                            left += crispCorr;
                            right += crispCorr;
                            boxPath = [
                                'M',
                                left, q3Plot,
                                'L',
                                left, q1Plot,
                                'L',
                                right, q1Plot,
                                'L',
                                right, q3Plot,
                                'L',
                                left, q3Plot,
                                'z'
                            ];

                        }

                        // The whiskers
                        if (whiskerLength) {
                            crispCorr = (whiskersAttr['stroke-width'] % 2) / 2;
                            highPlot = highPlot + crispCorr;
                            lowPlot = lowPlot + crispCorr;
                            whiskersPath = [
                                // High whisker
                                'M',
                                crispX - halfWidth * whiskerLength,
                                highPlot,
                                'L',
                                crispX + halfWidth * whiskerLength,
                                highPlot,

                                // Low whisker
                                'M',
                                crispX - halfWidth * whiskerLength,
                                lowPlot,
                                'L',
                                crispX + halfWidth * whiskerLength,
                                lowPlot
                            ];
                        }

                        // The median
                        crispCorr = (medianAttr['stroke-width'] % 2) / 2;
                        medianPlot = mathRound(point.medianPlot) + crispCorr;
                        medianPath = [
                            'M',
                            left,
                            medianPlot,
                            'L',
                            right,
                            medianPlot,
                            'z'
                        ];

                        //for inner box
                        if (doQuartiles) {

                            // inner Box for BoxPlot
                            innerBoxPath = [
                                'M',
                                left + 0.5, medianPlot + 0.5,
                                'L',
                                left + 0.5, q1Plot - 0.5,
                                'L',
                                right - 0.5, q1Plot - 0.5,
                                'L',
                                right - 0.5, medianPlot + 0.5,
                                'L',
                                left + 0.5, medianPlot + 0.5,
                                'z'
                            ];
                        }
                        // Create or update the graphics
                        if (graphic) { // update

                            point.stem.animate({d: stemPath});
                            if (whiskerLength) {
                                point.whiskers.animate({d: whiskersPath});
                            }
                            if (doQuartiles) {
                                point.box.animate({d: boxPath});
                                point.innerBox.animate({d: innerBoxPath});
                            }
                            point.medianShape.animate({d: medianPath});

                        } else { // create new
                            point.graphic = graphic = renderer.g()
                                .add(series.group).attr(graphicAttr);

                            point.stem = renderer.path(stemPath)
                                .attr(stemAttr)
                                .add(graphic);

                            if (whiskerLength) {
                                point.whiskers = renderer.path(whiskersPath)
                                    .attr(whiskersAttr)
                                    .add(graphic);
                            }
                            if (doQuartiles) {

                                point.box = renderer.path(boxPath)
                                    .attr(pointAttrNew)
                                    .add(graphic);

                                point.innerBox = renderer.path(innerBoxPath)
                                    .attr(innerPointAttr)
                                    .add(graphic);
                            }
                            point.medianShape = renderer.path(medianPath)
                                .attr(medianAttr)
                                .add(graphic);
                        }
                    }
                });

            });


            Highcharts.wrap(Highcharts.seriesTypes.boxplot.prototype, 'drawDataLabels', function (defaultCallback) {


                var series = this,
                    seriesOptions = series.options,
                    cursor = seriesOptions.cursor,
                    options = seriesOptions.dataLabels,
                    points = series.points,
                    pointOptions,
                    generalOptions,
                    str,
                    dataLabelsGroup;

                // Get the label name
                var showDataLabels = [
                    {
                        name: 'high',
                        enabled: true,
                        indent: -25,
                        path: function (x, y) {
                            return ['M', x, y, 'L', x + 8, y, x + 10, y - 5]
                        }
                    }, {
                        name: 'low',
                        enabled: true,
                        indent: 0,
                        path: function (x, y) {
                            return ['M', x, y, 'L', x + 8, y, x + 10, y + 5]
                        }
                    },
                    {
                        name: 'q1',
                        enabled: true,
                        indent: 0,
                        path: function (x, y) {
                            return ['M', x, y, 'L', x + 8, y, x + 10, y + 5]
                        }
                    },
                    {
                        name: 'q3',
                        enabled: true,
                        indent: -25,
                        path: function (x, y) {
                            return ['M', x, y, 'L', x + 8, y, x + 10, y - 5]
                        }
                    }, {
                        name: 'median',
                        enabled: true,
                        indent: -15,
                        path: function (x, y) {
                            return ['M', x, y, 'L', x + 8, y, x + 10, y]
                        }
                    }
                ];

                if (options.enabled) {

                    // Create a separate group for the data labels to avoid rotation
                    dataLabelsGroup = series.plotGroup(
                        'dataLabelsGroup',
                        'data-labels',
                        series.visible ? VISIBLE : HIDDEN,
                        options.zIndex || 6
                    );

                    /** SiSense extension - do not draw labels cause too much markers in the view */
                    var xMin = $$get(series, 'chart.xAxis.' + 0 + '.min'),
                        xMax = $$get(series, 'chart.xAxis.' + 0 + '.max'),
                        markerRadius = 5, //$$get(seriesOptions, 'marker.radius'), // 5 is small radius size
                        plotBoxWidth = series.chart.plotBox.width,
                        plotBoxHeight = series.chart.plotBox.height,
                        diameterMultiplier = 2,
                        spaceBetweenMarkersMultiplier = 2.5;


                    switch (series.type) {

                        case 'boxplot':


                            if (($$get(series, 'barW') < 20 || plotBoxHeight < 200)) {

                                if (defined($$get(points, 0 + '.dataLabel'))) {

                                    for (var i = 0, l = points.length; i < l; i++) {

                                        if (points[i].dataLabel) {
                                            // disable all datalabels if plotBoxHeight <100
                                            if ($$get(series, 'barW') < 20 || plotBoxHeight < 100) {
                                                points[i].dataLabel = points[i].dataLabel.destroy();
                                                return;
                                            } else {
                                                defined($$get(points[i].dataLabel, "high")) && points[i].dataLabel.high.destroy();
                                                defined($$get(points[i].dataLabel, "low")) && points[i].dataLabel.low.destroy();
                                            }

                                        }
                                    }
                                } else {
                                    if ($$get(series, 'barW') < 20 || plotBoxHeight < 100) {
                                        return;
                                    } else {
                                        delete showDataLabels[0];
                                        delete showDataLabels[1];
                                    }
                                }

                            }
                            break;

                        default:
                            break;
                    }
                    /** End of SiSense extension */

                        // Make the labels for each point
                    generalOptions = options;
                    each(points, function (point) {

                        var enabled,
                            dataLabel,
                            labelConfig,
                            attr,
                            name,
                            rotation,
                            connector = point.connector,
                            isNew = true,
                            shapeArgs = point.shapeArgs,
                            xLine = mathFloor(shapeArgs.x) + shapeArgs.width;

                        // Determine if each data label is enabled
                        pointOptions = point.options && point.options.dataLabels;
                        enabled = pick(pointOptions && pointOptions.enabled, generalOptions.enabled); // #2282


                        if (enabled) {
                            // Create individual options structure that can be extended without
                            // affecting others
                            options = merge(generalOptions, pointOptions);
                            rotation = options.rotation;

                            labelConfig = point.getLabelConfig();


                            //fix for drawTracker
                            if (!defined(point.dataLabel)) {
                                point.dataLabel = {};
                                point.dataLabel.element = {};
                                point.dataLabel.destroy = function () {
                                     //debugger;
                                    //TODO release memory
                                    /*removeEvent(wrapper, 'add', getSizeAfterAdd);

                                    // Added by button implementation
                                    removeEvent(wrapper.element, 'mouseenter');
                                    removeEvent(wrapper.element, 'mouseleave');

                                    if (text) {
                                        text = text.destroy();
                                    }
                                    if (box) {
                                        box = box.destroy();
                                    }
                                    // Call base implementation to destroy the rest
                                    SVGElement.prototype.destroy.call(wrapper);

                                    // Release local pointers (#1298)
                                    wrapper = renderer = updateBoxSize = updateTextPadding = boxAttr = getSizeAfterAdd = null;*/

                                }
                            }

                            each(showDataLabels, function (label) {

                                if (!label.enabled) {
                                    return;
                                }
                                //get data label for current value
                                dataLabel = point.dataLabel[label.name];

                                str = options.format ?
                                    format(options.format, labelConfig) :
                                    options.formatter.call(labelConfig, options, label.name);

                                // Determine the color
                                options.style.color = pick(options.color, options.style.color, series.color, 'black');

                                options.x = xLine + 15; //indent from box for text
                                options.y = (point[label.name + "Plot"] - point.highPlot) + label.indent;  //indent from box

                                // update existing label
                                if (dataLabel) {

                                    if (defined(str)) {
                                       /* dataLabel
                                            .attr({
                                                text: str
                                            });
                                        isNew = false;*/

                                    } else { // #1437 - the label is shown conditionally
                                        point.dataLabel[label.name] = dataLabel = dataLabel.destroy();
                                        if (connector) {
                                            point.connector = connector.destroy();
                                        }
                                    }

                                    // create new label
                                } else if (defined(str) && str !== 'null') { // second condition made by SiSense
                                    attr = {
                                        //align: align,
                                        fill: options.backgroundColor,
                                        stroke: options.borderColor,
                                        'stroke-width': options.borderWidth,
                                        r: options.borderRadius || 0,
                                        rotation: rotation,
                                        padding: options.padding,
                                        zIndex: 1
                                    };
                                    // Remove unused attributes (#947)
                                    for (name in attr) {
                                        if (attr[name] === UNDEFINED) {
                                            delete attr[name];
                                        }
                                    }

                                    dataLabel = point.dataLabel[label.name] = series.chart.renderer[rotation ? 'text' : 'label']( // labels don't support rotation
                                        str,
                                        0,
                                        -999,
                                        null,
                                        null,
                                        null,
                                        options.useHTML
                                    )
                                        .attr(attr)
                                        .css(extend(options.style, cursor && {cursor: cursor}))
                                        .add(dataLabelsGroup)
                                        .shadow(options.shadow);
                                }

                                if (dataLabel) {
                                    // Now the data label is created and placed at 0,0, so we need to align it
                                    series.alignDataLabel(point, dataLabel, options, null, isNew, label, dataLabelsGroup);

                                }
                            });
                        }
                    });
                }
            });


            /**
             * Align each individual data label
             */
            Highcharts.wrap(Highcharts.seriesTypes.boxplot.prototype, 'alignDataLabel', function (origFunc, point, dataLabel, options, alignTo, isNew ,label, dataLabelsGroup) {

                var chart = this.chart,
                    inverted = chart.inverted,
                    dlBox = point.dlBox || point.shapeArgs, // data label box for alignment
                    below = point.below || (point.plotY > pick(this.translatedThreshold, chart.plotSizeY)),
                    inside = pick(options.inside, !!this.options.stacking); // draw it inside the box?

                // Align to the column itself, or the top of it
                if (dlBox) { // Area range uses this method but not alignTo
                    alignTo = merge(dlBox);
                    alignTo.y += below ? alignTo.height : 0;
                    alignTo.height = 0;
                }

                // When alignment is undefined (typically columns and bars), display the individual
                // point below or above the point depending on the threshold
                options.align = pick(
                    options.align,
                    !inverted || inside ? 'center' : below ? 'right' : 'left'
                );
                options.verticalAlign = pick(
                    options.verticalAlign,
                    inverted || inside ? 'middle' : below ? 'top' : 'bottom'
                );

                //***
                var
                    bBox = dataLabel.getBBox(),
                    visible = this.visible && chart.isInsidePlot(point.plotX, point.plotY, inverted) && chart.isInsidePlot(point.plotX, point.yBottom, inverted),
                    alignAttr; // the final position;

                if (visible) {

                    // Add the text size for alignment calculation
                    extend(options, {
                        width: bBox.width,
                        height: bBox.height
                    });
                    // Allow a hook for changing alignment in the last moment, then do the alignment
                    alignAttr = {
                        align: options.align,
                        x: options.x, //alignTo.x + options.x + alignTo.width / 2,
                        y: alignTo.y + options.y + alignTo.height / 2
                    };
                    dataLabel[isNew ? 'attr' : 'animate'](alignAttr);

                    //Draw dataLabel lines
                    var y = point[label.name + "Plot"];

                    var pointAttr = point.pointAttr[point.selected ? 'select' : NORMAL_STATE];
                    xLine = mathFloor(dlBox.x) + dlBox.width;
                     var line = point.series.chart.renderer.path(label.path(xLine - 15, y))
                     .attr({
                     'stroke-width': 1,
                     'stroke': pointAttr['stroke'],
                     'stroke-opacity': pointAttr['fill-opacity']
                     })
                     .add(dataLabelsGroup);

                }

                // Show or hide based on the final aligned position
                if (!visible) {
                    dataLabel.attr({y: -999});
                }

            });

            //add functionality for the boxplot for set chartSize
            Highcharts.wrap(Highcharts.Chart.prototype, 'setChartSize', function (defaultCallback) {

                defaultCallback.apply(this);

                var chart = this;

                /* SiSense extension - bar chart - maximum xAxis categories */

                var chartType = $$get(chart, 'options.chart.type'),
                    plotLeft = mathRound(chart.plotLeft),
                    plotTop = mathRound(chart.plotTop),
                    plotWidth = mathMax(0, mathRound(chart.chartWidth - plotLeft - chart.marginRight)),
                    plotHeight = mathMax(0, mathRound(chart.chartHeight - plotTop - chart.marginBottom));
                //debugger;
                if (chartType === 'boxplot') {

                    var maxBarWidth = parseInt($$get(chart, 'options.xAxis.labels.style.fontSize')) || 20,
                        numCategories = $$get(chart, 'options.xAxis.categories.length'),
                        newNumCategories,
                        plotSize = plotWidth;

                    // too many ticks for bar chart, have to cut the xAxis categories
                    if (angular.isNumber(numCategories) && numCategories > 0 && plotSize / numCategories < maxBarWidth) {

                        newNumCategories = Math.round(plotSize / maxBarWidth);

                        // cut the xAxis categories
                        chart.options.xAxis.categories = chart.options.xAxis.categories.slice(0, newNumCategories);

                        // cut the series.data
                        for (var i = 0, l = chart.options.series.length; i < l; i++) {
                            chart.options.series[i].data = chart.options.series[i].data.slice(0, newNumCategories);
                        }

                        // push the error
                        if (angular.isArray($$get(chart, 'options.errors'))) {
                            chart.options.errors.push('ERROR_KEYS.tooManyTicks');
                        }

                        // remove xAxis labels and gridlines
                        $$set(chart, 'options.xAxis.labels.enabled', false);
                        $$set(chart, 'options.xAxis.gridLineWidth', 0);
                    }

                }

                /* end of SiSense extension */

            });

        }(Highcharts));


        //adds fill-opacity for highlight mode
        Highcharts.seriesTypes.boxplot.prototype.pointAttrToOptions['fill-opacity'] = 'fillOpacity';
        Highcharts.seriesTypes.boxplot.prototype.pointAttrToOptions['stroke-opacity'] = 'strokeOpacity';

    }]);
