mod.service('$data.plot', [

    'basicchart.services.$highchart.subtypesConverter',
    '$jaql',
    'widget-editor.services.$itemHelper',
    '$filter',
    'plugin-plotWidget.services.$plotHighchart',
    '$http',
    '$q',

    function ($subtypesConverter, $jaql, $itemHelper, $filter, $plotHighchart, $http, $q) {


        var self = this;

        var maxItemsDimensions = 1;


        // gets the widget data manifest
        this.manifest = function () {

            function resetDrillIfDimensionAdded(widget) {
                var drilledItem = widget.metadata.getDrilledItem();

                if (drilledItem && $itemHelper.isDimension(drilledItem)) {
                    var activeHierarchy = drilledItem.activeHierarchy,
                        hierarchies = drilledItem.hierarchies;

                    var rootItem = widget.metadata.drillUp(drilledItem, true);

                    if (activeHierarchy) {
                        rootItem.activeHierarchy = activeHierarchy;
                    }

                    if (hierarchies) {
                        rootItem.hierarchies = hierarchies;
                    }
                }
            }

            // defines the data values of all basic charts
            return {

                selection: ['category'],
                service: this,
                defaultQueryResult: {},
                panels: [
                    {
                        name: 'category',
                        type: 'default',
                        itemAdded: function (widget, item) {
                            // removing static color
                            if ($$get(item, "format.color.type") === "color" || $$get(item, "format.color.type") === "static") {
                                delete item.format.color;
                            }

                            if (defined(item, "format.color_bkp")) {
                                $jaql.resetColor(item);
                            }

                            if ($jaql.isMeasure(item) && defined(item, "format.members")) {
                                delete item.format.members;
                            }
                        },
                        metadata: {
                            types: ['dimensions'],
                            maxitems: 1
                        }
                    },
                    {
                        name: 'value',
                        type: 'default',
                        itemAdded: function (widget, item) {
                            // removing static color
                            if ($$get(item, "format.color.type") === "color" || $$get(item, "format.color.type") === "static") {
                                delete item.format.color;
                            }

                            if (defined(item, "format.color_bkp")) {
                                $jaql.resetColor(item);
                            }

                            if ($jaql.isMeasure(item) && defined(item, "format.members")) {
                                delete item.format.members;
                            }
                        },
                        metadata: {
                            types: ['dimensions'],
                            maxitems: 1
                        }

                    },
                    {
                        name: 'filters',
                        type: 'filters',
                        metadata: {
                            types: ['dimensions'],
                            maxitems: -1
                        }
                    }
                ]
            };
        };

        this.getItems = function (widget) {

            var result = [];

            // category
            window.sisenseUtils.array.copyTo(widget.metadata.panel(0).items, result);

            // measure
            window.sisenseUtils.array.copyTo(widget.metadata.panel(1).items, result);

            return result;
        };

        this.getMeasureFilteredDimension = function (widget) {

            var p = widget.metadata.panel(0);

            if (p.items.length === 0) {
                return;
            }

            return p.items[0].jaql.dim;
        };

        this.getMeasureFilteredItem = function (widget) {

            var p = widget.metadata.panel(0);

            if (p.items.length === 0) {
                return;
            }

            return $$.object.clone(p.items[0], true);
        };

        this.allocatePanel = function (widget, metadataItem) {

            if (!prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("value").items.length === 1) {
                return "value";
            }
            else if (!prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("category").items.length < maxItemsDimensions) {
                return "category";
            }
        };

        // returns true/ reason why the given item configuration is not/supported by the widget
        this.isSupported = function (items) {
            return this.rankMetadata(items, null, null) > -1;
        };

        // ranks the compatibility of the given metadata items with the widget
        this.rankMetadata = function (items, type, subtype) {
            var a = prism.$jaql.analyze(items);

            // requires 2 dimension and no measures
            if (a.measures.length === 0 && a.numericDimensions.length >= 1 && a.dimensions.length == 2) {

                return 0;
            }

            return -1;
        };

        // populates the metadata items to the widget
        this.populateMetadata = function (widget, items) {

            var a = prism.$jaql.analyze(items);
            var nonNumericDimension = _.filter(a.dimensions, function (dim) {
                return dim.jaql.datatype != 'numeric';
            });

            if (a.numericDimensions.length >= 2 && !nonNumericDimension.length) {
                widget.metadata.panel("category").push(a.numericDimensions[0]);
                widget.metadata.panel("value").push(a.numericDimensions[1]);
            } else {
                //use first non numeric dimension - text or date
                if (nonNumericDimension.length >= 1) {
                    widget.metadata.panel("category").push(nonNumericDimension[0]);
                }
                // allocating numeric value
                if (a.numericDimensions.length >= 1) {
                    widget.metadata.panel("value").push(a.numericDimensions[0]);
                }
            }

            widget.metadata.panel("filters").push(a.filters);
        };

        // builds a jaql query from the given widget
        this.buildQuery = function (widget) {

            // building jaql query object from widget metadata

            var value = widget.metadata.panel("value"),
                valueJaqls = _.pluck(value.items, 'jaql'),
                isNumeric = defined(_.findWhere(valueJaqls, {'datatype': 'numeric'}));

            var query = {
                datasource: widget.datasource,
                metadata: []
            };

            widget.metadata.panel("category").items.forEach(function (item) {
                query.metadata.push(item);
            });

            var enabledOutliers =  $$get(widget, 'style.outliers.enabled');
            //disable outliers if no category selected
            if (widget.metadata.panel("category").items.length === 0) {
                enabledOutliers = false;
            }
            //gets active whisker value

            var formula = getWhiskerFormula(widget.style.whisker);

            //create query from metadata
            if (defined(widget.metadata.panel("value"), 'items.0') && isNumeric) {
                addBoxQuartiles(query, widget.metadata.panel("value").items[0]);
                buildBoxplotJaql(query, widget.metadata.panel("value").items[0], formula,enabledOutliers);
            }

            // filters
            widget.metadata.panel("filters").items.forEach(function (item) {
                var filteritem = $$.object.clone(item, true);
                filteritem.panel = "scope";

                query.metadata.push(filteritem);
            });


            return query;

        };

        function getWhiskerFormula(whisker) {

            //  var whiskerFunc = iqr;
            var whiskerType,
                formula;

            _.each($$get(whisker), function (enabled, type) {
                if (enabled) {
                    var whiskerValueSplitted = type.split('/');
                    whiskerType = $$get(whiskerValueSplitted, 1);
                    switch (whiskerType) {
                        case "iqr":
                            //   whiskerFunc = iqr;
                            formula = ["(LOWERWHISKERMAX_IQR(low), all(low))", "(UPPERWHISKERMIN_IQR(high), all(high))", whiskerType];
                            break;
                        case "extremums":
                            //   whiskerFunc = extremums;
                            formula = ["(MIN(low), all(low))", "(MAX(high), all(high))", whiskerType];
                            break;
                        case "deviation":
                            //  whiskerFunc = deviation;
                            formula = ["(LOWERWHISKERMAX_STDEVP(low), all(low))", "(UPPERWHISKERMIN_STDEVP(high), all(high))", whiskerType];
                            break;
                    }
                    return;
                }
            });
            return formula;
        }

        // prepares the widget-specific query result from the given result data-table
        this.processResult = function (widget, queryResult) {

            var queryResultRows = queryResult.rows();
            var i = 0, l = queryResultRows.length;
            var countOutliers = 0,
                outliers = [];

            for (; i < l; i++) {
                countOutliers += $$get(queryResultRows[i],"6.data");
            }

            var outliers = [];
            //send extra queries for the outliers only if outliers.enabled = true or countOutliers less than 20000
            //temorary fix due to backend issue, always enabled
            countOutliers = 0;
            if ($$get(widget, 'style.outliers.enabled') && countOutliers < 20000) {

                var lowerOutlier = buildOutliersJaql(widget, "low");
                var upperOutliersJaql = buildOutliersJaql(widget, "high");
                var ds = widget.datasource;
                var dsTitle = typeof ds == 'string' ? ds : ds.title;

                /** SisenseJS.String.format */
                var url = window.sisenseUtils.string.format('/api/datasources/{0}/jaql', encodeURIComponent(dsTitle));

                var result = $http({method: 'POST', url: url, data: angular.toJson(lowerOutlier)})
                    .then(function (response) {

                        // verifying incoming response
                        if (!defined(response, "data") || response.status !== 200) {

                            throw response;
                        }

                        //add lower outliers
                        $.merge(outliers, response.data.values);

                        //send upper query
                        return $http({method: 'POST', url: url, data: angular.toJson(upperOutliersJaql)})
                    })
                    .then(function (response) {

                        // verifying incoming response
                        if (!defined(response, "data") || response.status !== 200) {

                            throw response;
                        }

                        //add upper outliers
                        $.merge(outliers, response.data.values);

                        //run processResult + outliers data
                        var options = processResultBoxplot(widget, queryResult, true, outliers);

                        return options;
                    });

                return result
            } else {

                //run original boxplot function without outliers data
                return processResultBoxplot(widget, queryResult, false);
            }

            //boxplot process result data
            function processResultBoxplot(widget, queryResult, enabledOutliers, outliers) {

                var familyType = 'plotwidget';

                //use original findTitles function
                var titlesForChart = $plotHighchart.findTitles(widget, familyType, queryResult.metadata()),
                    hasSelection = _.some(titlesForChart.category, function (category) {
                        return (category.affectFromGlobalFilter === true && category.filterAll !== true);
                    });

                // getting basic options
                var options = $plotHighchart.options(familyType, titlesForChart);

                // getting css style for chart // should be included to the
                var dashboardstyle = generateCSS();


                // generate legend object from the given style/ template
                options.legend = $plotHighchart.legend(widget, titlesForChart, familyType, $$get(dashboardstyle, 'children.legend.style'), $$get(widget, 'style.legend'));


                // generate chart object from the given style/ template
                options.chart = $plotHighchart.chart($$get(dashboardstyle, 'style'), $$get(widget, 'style.chart'), widget.subtype, $$get(options, '$familyType'), widget, titlesForChart);

                // generate xaxis legend object from the given style/ template
                var x2TitleLabel = {};
                options.xAxis = $plotHighchart.xaxis(widget, $$get(dashboardstyle, 'children.axes'), $$get(widget, 'style.xAxis'), titlesForChart, x2TitleLabel, familyType);

                // generate yaxis (Major and Minors) legend object from the given style/ template
                options.yAxis = $plotHighchart.yaxis($$get(dashboardstyle, 'children.axes'), $$get(widget, 'style.yAxis'), $$get(widget, 'style.y2Axis'), titlesForChart, familyType);

                //widget sub
                // generate style of the plot charts, spline, stacked and normal
                options.plotOptions = $plotHighchart.boxPlotOptions(titlesForChart, $$get(dashboardstyle, 'children.series.children'), $$get(widget, 'style.seriesLabels'),
                    $$get(widget, 'style.markers'), $$get(widget, 'style.lineWidth.width'), $$get(options, 'chart.backgroundColor'), hasSelection, null, $$get(options, 'chart.type'), widget);

                // generate series object for cartesian charts from the given style/ template
                options.series = $plotHighchart.boxplotSeriesAndCategories(titlesForChart, queryResult, options.xAxis, options.plotOptions, $$get(options, 'chart.type'), options.navigator, $$get(widget, 'style.dataLimits'), $$get(options, 'errors'), widget, enabledOutliers, outliers);

                // generate tooltip options object from the given style/ template
                options.tooltip = $plotHighchart.tooltip(widget, titlesForChart, queryResult, $$get(dashboardstyle, 'children.tooltip.style'), $$get(widget, 'style.tooltip'), '', options.plotOptions, options.series, familyType);

                //bird's eye for cartesian charts
                options.navigator = $plotHighchart.navigator(widget, $$get(widget, 'style.navigator'), $$get(options, 'chart.type'), options.series, $$get(widget, 'style.xAxis'));


                // adds style to axes from css, numbers at the end represents axis in binary:
                var axisEnabled =                                                           // Binary Flags Values
                    ($$get(widget, 'style.xAxis.enabled') == true ? 1 : 0)                  // x-Axis enabled
                    + ($$get(widget, 'style.xAxis.ticks') == true ? 2 : 0)                  // x ticks enabled
                    + ($$get(widget, 'style.xAxis.labels.enabled') == true ? 4 : 0)         // x labels enabled
                    + ($$get(widget, 'style.xAxis.gridLines') == true ? 8 : 0)              // x gridlines enabled
                    + ($$get(widget, 'style.yAxis.enabled') == true ? 16 : 0)               // y1-Axis enabled
                    + ($$get(widget, 'style.yAxis.ticks') == true ? 32 : 0)                 // y1 tick enabled
                    + ($$get(widget, 'style.yAxis.labels.enabled') == true ? 64 : 0)        // y1 labels enabled
                    + ($$get(widget, 'style.yAxis.gridLines') == true ? 128 : 0);          // y1 gridlines enabled


                // adds css style for the chart axes
                $plotHighchart.addCssStyleForAxes($$get(dashboardstyle, 'children.axes'), options, axisEnabled);

                // adds css style for the chart axes
                $plotHighchart.addCssStyleForAxes($$get(dashboardstyle, 'children.x-axis'), options, axisEnabled);

                // adds css style for the chart axes
                $plotHighchart.addCssStyleForAxes($$get(dashboardstyle, 'children.y-axis'), options, axisEnabled);

                return options;
            }
        };

        // function adds 1,3 median quartile
        function addBoxQuartiles(query, item) {

            var q1Jaql = {
                "type": "measure",
                "formula": "(QUARTILE([q1], 1), all(q1))",
                "context": {
                    "[q1]": $$.object.clone(item.jaql, true)
                },
                "title": "q1"
            };
            var q3Jaql = {
                "type": "measure",
                "formula": "(QUARTILE([q3], 3), all(q3))",
                "context": {
                    "[q3]": $$.object.clone(item.jaql, true)
                },
                "title": "q3"
            };
            var medianJaql = {
                "type": "measure",
                "formula": "(MEDIAN([med]), all(med))",
                "context": {
                    "[med]": $$.object.clone(item.jaql, true)
                },
                "title": "median"
            };

            query.metadata.push({"jaql": q1Jaql, format: item.format, $$panel: item.$$panel, $$guid: item.$$guid});
            query.metadata.push({"jaql": medianJaql, format: item.format, $$panel: item.$$panel, $$guid: item.$$guid});
            query.metadata.push({"jaql": q3Jaql, format: item.format, $$panel: item.$$panel, $$guid: item.$$guid});


        }

        function createCountJaql(query, fromNotEqual, toNotEqual, item) {

            var countJaql = {
                "type": "measure",
                "formula": "([main], [filter])",
                "title": "countOutliers",
                "context": {
                    "[main]": {
                        "agg": "countduplicates",

                        "table": item.jaql.table,
                        "column": item.jaql.column,
                        "dim": item.jaql.dim,
                        "datatype": item.jaql.datatype
                    },
                    "[filter]": {
                        "table": item.jaql.table,
                        "column": item.jaql.column,
                        "dim": item.jaql.dim,
                        "datatype": item.jaql.datatype,
                        "filter": {
                            "or": [{
                                "fromNotEqual": fromNotEqual
                            }, {
                                "toNotEqual": toNotEqual
                            }
                            ],
                            "custom": true
                        }
                    }
                }
            };

            query.metadata.push({"jaql": countJaql, $$panel: item.$$panel, $$guid: item.$$guid});
        }

        //function adds the interquartile range values.
        function buildBoxplotJaql(query, item, formula, enabledOutliers) {

            var itemJaql = $$.object.clone(item.jaql, true);

            var minJaql = {
                "type": "measure",
                "formula": formula[0],
                "context": {
                    "[low]": itemJaql
                },
                "title": "low"
            };
            var maxJaql = {
                "type": "measure",
                "formula": formula[1],
                "context": {
                    "[high]": itemJaql
                },
                "title": "high"
            };

            query.metadata.push({"jaql": minJaql, format: item.format, $$panel: item.$$panel, $$guid: item.$$guid});
            query.metadata.push({"jaql": maxJaql, format: item.format, $$panel: item.$$panel, $$guid: item.$$guid});

            if(formula[2] !== 'exterums' && enabledOutliers) {
                //add jaql for the count
                //temp disabed due to issues in backend part
               //createCountJaql(query, maxJaql,minJaql, item);
            }

            return [minJaql, maxJaql];
        }

        function buildOutliersJaql(widget, type) {

            var query = {
                datasource: widget.datasource,
                metadata: []
            };

            widget.metadata.panel("category").items.forEach(function (item) {
                query.metadata.push(item);
            });

            var formula = getWhiskerFormula(widget.style.whisker);
            var item = widget.metadata.panel("value").items[0];
            var itemJaql = $$.object.clone(item.jaql, true);

            switch (type) {

                case "low":
                    itemJaql.filter = {
                        "toNotEqual": {
                            "formula": formula[0],
                            "context": {
                                "low": $$.object.clone(item.jaql, true)
                            }
                        },
                        "custom": true
                    };

                    break;

                case "high":

                    itemJaql.filter = {
                        "fromNotEqual": {
                            "formula": formula[1],
                            "context": {
                                "high": $$.object.clone(item.jaql, true)
                            }
                        },
                        "custom": true
                    };
                    break;
            }
            query.metadata.push({"jaql": itemJaql});

            //adds filters to the outliers jaql
            widget.metadata.panel("filters").items.forEach(function (item) {

                var filteritem = $$.object.clone(item, true);
                filteritem.panel = "scope";

                query.metadata.push(filteritem);
            });


            return query;
        }

        //TODO use css file for creation
        function generateCSS() {

            var css = {
                "style": {},
                "children": {
                    "series": {
                        "style": {},
                        "children": {
                            "labels": {
                                "style": {
                                    "font-style": "normal",
                                    "font-variant": "normal",
                                    "font-weight": "normal",
                                    "font-size": "13px",
                                    "line-height": "normal",
                                    "font-family": "ubuntulight, sans-serif",
                                    "color": "#56535b"
                                },
                                "children": {},
                                "states": {}
                            }
                        },
                        "states": {}
                    },
                    "axes": {
                        "style": {
                            "border-top-color": "#C5C5C5",
                            "border-right-color": "#C5C5C5",
                            "border-bottom-color": "#C5C5C5",
                            "border-left-color": "#C5C5C5",
                            "border-top-width": "1px",
                            "border-right-width": "1px",
                            "border-bottom-width": "1px",
                            "border-left-width": "1px"
                        },
                        "children": {
                            "ticks": {
                                "style": {},
                                "children": {},
                                "states": {
                                    "major": {
                                        "style": {
                                            "border-top-color": "#C5C5C5",
                                            "border-right-color": "#C5C5C5",
                                            "border-bottom-color": "#C5C5C5",
                                            "border-left-color": "#C5C5C5"
                                        },
                                        "children": {},
                                        "states": {}
                                    },
                                    "minor": {
                                        "style": {
                                            "border-top-color": "#C5C5C5",
                                            "border-right-color": "#C5C5C5",
                                            "border-bottom-color": "#C5C5C5",
                                            "border-left-color": "#C5C5C5"
                                        },
                                        "children": {},
                                        "states": {}
                                    }
                                }
                            },
                            "labels": {
                                "style": {
                                    "font-style": "normal",
                                    "font-variant": "normal",
                                    "font-weight": "normal",
                                    "font-size": "13px",
                                    "line-height": "normal",
                                    "font-family": "ubuntulight, sans-serif",
                                    "color": "#56535b"
                                },
                                "children": {},
                                "states": {}
                            },
                            "gridlines": {
                                "style": {
                                    "border-top-color": "#C5C5C5",
                                    "border-right-color": "#C5C5C5",
                                    "border-bottom-color": "#C5C5C5",
                                    "border-left-color": "#C5C5C5",
                                    "border-top-style": "dotted",
                                    "border-right-style": "dotted",
                                    "border-bottom-style": "dotted",
                                    "border-left-style": "dotted"
                                },
                                "children": {},
                                "states": {}
                            },
                            "titles": {
                                "style": {
                                    "font-style": "normal",
                                    "font-variant": "normal",
                                    "font-weight": "normal",
                                    "font-size": "13px",
                                    "line-height": "normal",
                                    "font-family": "ubuntulight, sans-serif",
                                    "color": "#2c3342"
                                },
                                "children": {},
                                "states": {}
                            }
                        },
                        "states": {}
                    },
                    "legend": {
                        "style": {
                            "background-color": "transparent",
                            "border-top-width": "0px",
                            "border-right-width": "0px",
                            "border-bottom-width": "0px",
                            "border-left-width": "0px",
                            "border-top-color": "transparent",
                            "border-right-color": "transparent",
                            "border-bottom-color": "transparent",
                            "border-left-color": "transparent",
                            "font-style": "normal",
                            "font-variant": "normal",
                            "font-weight": "normal",
                            "font-size": "13px",
                            "line-height": "normal",
                            "font-family": "ubuntulight, sans-serif",
                            "color": "#56535b"
                        },
                        "children": {},
                        "states": {}
                    },
                    "tooltip": {
                        "style": {
                            "font-style": "normal",
                            "font-variant": "normal",
                            "font-weight": "normal",
                            "font-size": "16px",
                            "line-height": "normal",
                            "font-family": "Abel, sans-serif",
                            "color": "#56535b"
                        },
                        "children": {},
                        "states": {}
                    }
                },
                "states": {}
            };
            return css;
        }
    }]);