﻿//
// common highcharts wrapper service

mod.service('$plotHighchart', [
    '$jaql',
    'dashboard-base.services.$mask',
    'plugin-plotWidget.services.$plotHighchart.series', // new directive
    'chartsGenerator.services.$chartGenerator.tooltip',
    'plugin-plotWidget.services.$plotHighchart.tooltip',  // new directive
    'basicchart.services.$highchart.selections',
    'chartsGenerator.services.$chartGenerator.cssStyling',
    'basicchart.services.$highchart.metadataManipulation',
    'basicchart.services.$highchart.menus',
    'basicchart.services.$highchart.draggingAxis',
    'basicchart.services.$highchart.stylerUpdate',
    '$filter',
    'ux-controls.services.$dom',
    '$dashboard',
    '$rootScope',
    '$device',
    'base.services.$mongo',
    'base.services.reportingConfig',

    function ($jaql, $mask, $plotHighchartSeries, $highchartTooltip, $plotHighchartTooltip, $highchartSelections, $highchartCssStyling, $metadataManipulate, $menus, $draggingAxis, $stylerUpdate, $filter, $dom, $dashboard, $rootScope, $device, $mongo, reportingConfig) {

        var self = this;
        //
        // DEFAULT SETTINGS

        // holds all default setting to create the highcharts
        var default_chart_settings = Object.freeze({

            //
            // CARTESIAN SETTINGS

            minimumWidthToShowXAxisGridLines: 300,
            minimumHeightToShowYAxisGridLines: 300,

            minimumWidthToShowYAxisTitles: 300,
            minimumHeightToShowXAxisTitles: 300,

            minSpacePerItemHorizonalLabelXAxis: 80,
            minSpacePerItemVerticalLabelXAxis: 40,

            navigatorScrollMinSpacePerItem: 20,

            minSpacePerItemVerticalLabelXAxisBarChart: 40,

            heightConvertScrollbarToNavigator: 400,
            extraHeightToConvertToBigNavigator: 200,
            navigatorSmallHeight: 42,
            navigatorBigHeight: 84,

            seriesCapacity: 100000,
            categoriesCapacity: 100000,
            categoriesCapacityYaxis: 100000,

            minHeightToActivateScrollbar: 200,
            minWidthToActivateScrollbar: 400,

            navigatorsScrollbar: {
                enabled: true,
                barBackgroundColor: 'transparent',
                barBorderRadius: 0,
                barBorderWidth: 0,
                buttonBackgroundColor: 'transparent',
                buttonBorderWidth: 0,
                buttonBorderRadius: 0,
                trackBackgroundColor: 'none',
                trackBorderWidth: 0,
                trackBorderRadius: 0,
                trackBorderColor: 'transparent',
                buttonArrowColor: 'transparent',
                height: 2

            },

            normalScrollBar: {
                enabled: true,
                barBackgroundColor: '#E6E6E6',
                barBorderRadius: 3,
                barBorderWidth: 0,
                buttonBackgroundColor: 'transparent',
                buttonBorderWidth: 0,
                buttonArrowColor: 'transparent',
                buttonBorderRadius: 7,
                rifleColor: 'transparent',
                trackBackgroundColor: 'transparent',
                trackBorderWidth: 1,
                trackBorderColor: 'transparent',
                trackBorderRadius: 7,
                height: 8
            },

            //
            // PIE SETTINGS

            // labels percentages
            pieMinimumSliceAngle: 3,
            pieMinNumOfSlicesToActivateOthers: 8,
            pieMinimumFontSizeToTextLabel: 8,

            // element height to hide the slices labels
            pieElementHeightHideLabels: 250,

            // element width to hide the slices labels
            pieElementWidthHideLabels: 350,


            //
            // LEGEND SETTINGS

            legendSettings: {

                // element height to hide the legend
                elementHeightHideLegend: 250,

                // element width to hide the legend
                elementWidthHideLegend: 250
            },

            //
            // TOOLTIP SETTINGS

            tooltipSettings: {

                cartesian: {

                    cartesianShowMultiSeries: 4
                }
            },

            // errors keys
            exceededSeriesCapacity: 'ERROR_KEYS.exceededSeriesCapacity',
            exceededCategoriesCapacity: 'ERROR_KEYS.exceededCategoriesCapacity',
            axisExceededCategoriesCapacity: 'ERROR_KEYS.axisExceededCategoriesCapacity',
            tooManyPieSlices: 'ERROR_KEYS.tooManyPieSlices',
            allSlicesZeroValues: 'ERROR_KEYS.allSlicesZeroValues'

        });

        //
        // PRIVATES

        //
        this.boxPlotOptions = function (titlesForChart, style, templateLabels, templateMarkers, templateLineWidth, chartBackgroundColor, hasSelection, fullType, chartType, widget) {
            var formatWithCommas = $filter('number');
            //dataLabels not supported yet
            var op = {
                series: {
                    dataLabels: {
                        enabled: false,
                        style: {},
                        formatter: function (el, pointLabel) {
                            var value;
                            var func = $$get(this, 'series.options.mask');

                            if ($$get(titlesForChart, 'value.0.isNumericDimension')) { //&& !titlesForChart.yAxis[0].isDummy) {
                                value = this.point[pointLabel];
                            }

                            return defined(value) ? func(value) : formatWithCommas(value);
                        }
                    },
                    events: {legendItemClick: false}
                },
                stickyTracking: false  // parameter
            };

            // sets series labels template
            if (defined(templateLabels)) {

                if (defined(templateLabels.enabled)) {
                    op.series.dataLabels.enabled = templateLabels.enabled;
                }

                if (defined(templateLabels.rotation)) {
                    op.series.dataLabels.rotation = templateLabels.rotation;
                }
            }

            // sets style for series from css style
            if (defined(style)) {

                // labels
                if (defined(style.labels)) {

                    // generate FONT style
                    $.extend(op.series.dataLabels.style, $highchartCssStyling.generateFontStyle($$get(style, 'labels.style')));

                    // generate fill style
                    $.extend(op.series.dataLabels, $highchartCssStyling.generateFillStyle($$get(style, 'labels.style')));

                    // generate border style
                    $.extend(op.series.dataLabels, $highchartCssStyling.generateBorderStyle($$get(style, 'labels.style')));

                }

            }
            return op;

        };

        //sets element if not exists and returns it
        var $$setNget = function (element, path) {

            var pointerResult = element;

            if (!path) {
                return element;
            }

            var pathSplitted = path.split('.'), nextOnPath;

            var i = 0, l = pathSplitted.length;
            for (; i < l; i++) {

                nextOnPath = pathSplitted[i];

                if (!pointerResult[nextOnPath]) {
                    pointerResult[nextOnPath] = {};
                }

                pointerResult = pointerResult[nextOnPath];
            }

            return pointerResult;
        };

        // Remove Axis titles due to element size
        var disableTitlesDueToElementSize = function disableTitlesDueToElementSize($lmnt, queryResult) {

            var elementWidth = $lmnt.width(),
                elementHeight = $lmnt.height(),
                chartType = queryResult.chart.type;

            var xRemoveTitles = chartType !== 'bar' ? elementHeight < default_chart_settings.minimumHeightToShowXAxisTitles : elementWidth < default_chart_settings.minimumWidthToShowYAxisTitles,
                yRemoveTitles = chartType !== 'bar' ? elementWidth < default_chart_settings.minimumWidthToShowYAxisTitles : elementHeight < default_chart_settings.minimumHeightToShowXAxisTitles;

            // hide / show xAxis titles
            if (xRemoveTitles) {

                $$set($$get(queryResult, 'xAxis'), 'title.text', undefined);

                var labels = $$get(queryResult, 'labels');

                if (labels) {
                    labels.items = _.reject(labels.items, function (label) {
                        return label.labelType === 'axisTitle';
                    });
                }
            }

            // hide / show yAxis titles
            if (yRemoveTitles) {
                _.each($$get(queryResult, 'yAxis'), function (axis) {
                    $$set(axis, 'title.text', undefined);
                });
            }
        };

        // sets AxisSetting of both axis according element size
        var setAxisAndScrollbarSettingDueToElementSize = function (widget, xAxis, yAxis, navigator, series, plotOptions, chartType, isPolar, prevScrollerLocation, resetScroller, $lmnt, element, renderMode) {

            // element validation
            if (!$lmnt || series.length === 0) {
                return;
            }

            var elementWidth = $lmnt.width(), elementHeight = $lmnt.height();
            //elementHeight = element.height ? parseInt(element.height, 10) : $lmnt.height();

            // returns to editor decision on navigator when exitting wizard new widget mode
            if (widget.appstate !== 'newwidget' && $$get(navigator, 'enabledFromEditor') !== $$get(navigator, 'enabled')) {
                $$set(navigator, 'enabled', $$get(navigator, 'enabledFromEditor'));
            }

            if (renderMode || elementHeight < default_chart_settings.minHeightToActivateScrollbar) {
                $$set(navigator, 'enabled', false);
            }

            if ($$get(navigator, 'enabled') === true) {

                var rotation = $$get(prevScrollerLocation, 'rotation');

                switch (rotation) {

                    case 0:
                        $.extend(xAxis.labels, {rotation: 0, align: 'center', x: 0, y: 20});
                        break;

                    case 90:
                        $.extend(xAxis.labels, {rotation: 90, align: 'left', x: -4, y: 20});
                        break;

                    default:
                        $.extend(xAxis.labels, {rotation: rotation});
                }

                var min = $$get(prevScrollerLocation, 'min'),
                    max = $$get(prevScrollerLocation, 'max');

                // fix pris-10956 - bug created somehow cause of the continuous time...
                // max and sometimes even min values are greater than the categories length..
                if (max >= $$get(xAxis, 'categories.length')) {
                    min = null;
                    max = null;
                }

                /*
                 // In case the navigator is not yet defined
                 if (!min && !max)
                 {
                 var items = widget.items();
                 if (_.find(items, function(i) {
                 if (i.jaql.agg) return false;
                 return i.jaql.datatype == "datetime";
                 }))
                 {
                 $$set(xAxis, 'containsDate', true);
                 }
                 }
                 */

                // restore scroller location
                if (angular.isNumber(min) && angular.isNumber(max)) {

                    $$set(xAxis, 'min', min);
                    $$set(xAxis, 'max', max);
                }

            }

            // reset scroller location
            if (resetScroller) {

                $$set(xAxis, 'min', null);
                $$set(xAxis, 'max', null);

                // reset the scroller location in the widget memory for refreshes
                $$set(widget, 'options.previousScrollerLocation', {min: null, max: null});
                widget.changesMade('scroll');

            }

            // chart states to get the yAxis min and max
            var isPercent = $$get(plotOptions, 'column.stacking') === 'percent' || $$get(plotOptions, 'bar.stacking') === 'percent' || $$get(plotOptions, 'area.stacking') === 'percent' || $$get(plotOptions, 'areaspline.stacking') === 'percent';
            var isStacking = $$get(plotOptions, 'column.stacking') === 'normal' || $$get(plotOptions, 'bar.stacking') === 'normal' || $$get(plotOptions, 'area.stacking') === 'normal' || $$get(plotOptions, 'areaspline.stacking') === 'normal';
            var hasExplicitType;


            // if there is no break by, we can have two yAxis and handle it
            if (defined($$get(series, 0 + '.yAxis'))) {

                var seriesY0 = _.filter(series, function (ser) {
                    return ser.yAxis === 0;
                });
                hasExplicitType = defined(_.find(seriesY0, function (ser) {
                    return defined(ser.type);
                }));
                setAxisMinAndMax(seriesY0, $$get(yAxis, 0), 'y', isPercent, isStacking, hasExplicitType, chartType);

                if (seriesY0.length !== series.length) {

                    var seriesY1 = _.filter(series, function (ser) {
                        return ser.yAxis === 1;
                    });
                    hasExplicitType = defined(_.find(seriesY1, function (ser) {
                        return defined(ser.type);
                    }));
                    //    setAxisMinAndMax(seriesY1, $$get(yAxis, 1), 'y', isPercent, isStacking, hasExplicitType, chartType);
                }
            }

            // else we can have only left yAxis
            else {

                setAxisMinAndMax(series, $$get(yAxis, 0), 'y', isPercent, isStacking, hasExplicitType, chartType);
            }


            // set min max to xAxis (relevant for scatter chart)
            if (chartType === 'bubble') {
                setAxisMinAndMax(series, xAxis, 'x', null, null, null, chartType);
            }

            // Remove whole or part of the Axis due to element size

            var xRemoveLabelsAndTicks = chartType !== 'bar' ? elementHeight < 180 : elementWidth < 180, xRemoveAxis = chartType !== 'bar' ? elementHeight < 120 : elementWidth < 120;
            var yRemoveLabelsAndTicks = chartType !== 'bar' ? elementWidth < 180 : elementHeight < 180, yRemoveAxis = chartType !== 'bar' ? elementWidth < 120 : elementHeight < 120;

            // hide / show xAxis elements

            // disable xAxis due to element small width
            if (xRemoveLabelsAndTicks) {

                if (xRemoveAxis) {
                    $$set(xAxis, 'lineWidth', 0);
                    $$set(xAxis, 'lineColor', 'transparent');
                }

                $$set(xAxis, 'minorTickLength', 0);
                $$set(xAxis, 'tickLength', 0);
                $$set(xAxis, 'labels.enabled', false);

            }

            // hide / show yAxis elements

            // disable yAxis due to element small width
            if (yRemoveLabelsAndTicks) {

                for (var i = 0, l = yAxis.length; i < l; i++) {

                    var currentYAxis = $$get(yAxis, i);

                    if (yRemoveAxis) {
                        $$set(currentYAxis, 'lineWidth', 0);
                        $$set(currentYAxis, 'lineColor', 'transparent');
                    }

                    $$set(currentYAxis, 'minorTickLength', 0);
                    $$set(currentYAxis, 'tickLength', 0);
                    $$set(currentYAxis, 'labels.enabled', false);
                }

            }

            // removes y-axis gridlines if the container height is too small
            if (elementHeight < default_chart_settings.minimumHeightToShowYAxisGridLines) {

                // runs over all yAxis (main and secondary)
                for (var i = 0, l = yAxis.length; i < l; i++) {

                    $$set(yAxis, i + '.gridLineWidth', 0);
                    $$set(yAxis, i + '.minorGridLineWidth', 0);
                }
            }

            // removes x-axis gridlines if the container width is too small
            if (elementWidth < default_chart_settings.minimumWidthToShowXAxisGridLines) {

                $$set(xAxis, 'gridLineWidth', 0);
                $$set(xAxis, 'minorGridLineWidth', 0);
            }


            if (chartType !== 'bubble' && (elementWidth < default_chart_settings.minWidthToActivateScrollbar || elementHeight < default_chart_settings.minHeightToActivateScrollbar)) {

                $$set(navigator, 'enabled', false);

                //$$set(xAxis, 'labels.style', { color: 'transparent' });

                $$set(xAxis, 'min', 0);

                if (defined(xAxis.max)) {
                    delete xAxis.max;
                }

                if (defined(xAxis.plotBands)) {
                    delete xAxis.plotBands;
                }

                if (defined(xAxis.plotLines)) {
                    delete xAxis.plotLines;
                }
            }
        };

        // change legend setteings due to container width and height
        var changeLegendSettingDueToElementSize = function (legend, $lmnt, piePlotOptions) {

            // element validation
            if (!$lmnt) {
                return;
            }

            var elementWidth = $lmnt.width(), elementHeight = $lmnt.height();

            // removes the legend if needed
            if ($$get(legend, 'enabled') === true) {

                // the legend top or bottom, will have to check height
                if ($$get(legend, 'align') === 'center') {

                    if (elementHeight < $$get(default_chart_settings, 'legendSettings.elementHeightHideLegend')) {

                        $$set(legend, 'enabled', false);
                    }
                }

                // the legend on the left or right side
                else {

                    if (elementWidth < $$get(default_chart_settings, 'legendSettings.elementWidthHideLegend')) {

                        $$set(legend, 'enabled', false);
                    }
                }
            }

            // removes the slices labels if needed
            if ($$get(piePlotOptions, 'dataLabels.enabled') === true) {

                if (elementHeight < default_chart_settings.pieElementHeightHideLabels || elementWidth < default_chart_settings.pieElementWidthHideLabels) {

                    $$set(piePlotOptions, 'dataLabels.enabled', false);
                }
            }


        };

        //calculate and set min and max for single axis
        var setAxisMinAndMax = function (series, axis, xOrY, isPercent, isStacking, hasExplicitType, chartType) {

            // validate for numeric axis
            if ($$get(axis, 'categories.length') > 0) {
                return;
            }

            if (chartType == 'boxplot') {
                xOrY = 'low';
            }
            var autoMin = null,
                autoMax = null;

            // set y Axis labels min max
            var min, max;

            if (isPercent) {

                autoMin = 0;
                autoMax = 100;
            }

            else if (isStacking) {

                var minMaxStack = {};

                for (var i = 0, l = (_.max(series, function (ser) {
                    return ser.data.length;
                })).data.length; i < l; i++) {
                    minMaxStack[i] = {minus: 0, plus: 0}
                }

                var y;

                for (var i = 0, l = series.length; i < l; i++) {

                    for (var j = 0, m = series[i].data.length; j < m; j++) {

                        y = $$get(series, i + '.data.' + j + '.' + xOrY);

                        if (defined(y)) {

                            if (y >= 0) {
                                minMaxStack[j].plus += y;
                            }
                            else {
                                minMaxStack[j].minus += y;
                            }
                        }
                    }
                }

                autoMin = (_.min(minMaxStack, function (x) {
                    return x.minus;
                })).minus;
                autoMax = (_.max(minMaxStack, function (x) {
                    return x.plus;
                })).plus;

            }

            else {
                //if (hasExplicitType || (!isPercent && !isStacking)) {

                min = $$get(series, '0.data.0.' + xOrY) || 0, max = min;
                var value;

                _.each(series, function (ser) {
                    _.each(ser.data, function (point) {

                        value = $$get(point, xOrY);

                        if (defined(value)) {

                            if (value > max) {
                                max = value;
                            }

                            if (value < min) {
                                min = value;
                            }
                        }
                    })
                });

                //calculate max for boxplot
                if (chartType == 'boxplot') {

                    var maxName = 'high';
                    max = $$get(series, '0.data.0.' + maxName) || 0;

                    _.each(series, function (ser) {
                        _.each(ser.data, function (point) {

                            value = $$get(point, maxName);

                            if (defined(value)) {
                                if (value > max) {
                                    max = value;
                                }
                            }
                        })
                    });
                }

                autoMin = min > 0 && chartType !== 'bubble' ? 0 : min;
                autoMax = max > 0 ? max : 0;
            }

            var isAxisLog = $$get(axis, 'type') === 'logarithmic';

            // logarithmic axis can't be with min value of zero or less (highcharts error #10)
            if (isAxisLog) {

                if (autoMax <= 0) {
                    $$set(axis, 'type', 'linear');
                }

                else if (autoMin <= 0) {
                    autoMin = 1;
                }

            }

            // fix position for yAxis single point bubble chart to the middle of the axis
            if (chartType === 'bubble' && autoMin === autoMax && angular.isNumber(autoMax)) {
                if (isAxisLog) {
                    $$set(axis, 'type', 'linear');
                }

                autoMin -= 1;
                autoMax += 1;

            }

            // set the minimum and maximum of the axis if were not defined from the styler
            var isGivenMinNotNumber = !angular.isNumber(axis.min),
                isGivenMaxNotNumber = !angular.isNumber(axis.max);

            if (isGivenMinNotNumber && isGivenMaxNotNumber) {
                axis.min = autoMin;
                axis.max = autoMax;
            }

            // min xor max (only one of them) is defined in the styler, and given min is greater the the max value of the chart or the vice versa
            // ^ = xor (only one of the two variables defined)
            else if (isGivenMinNotNumber ^ isGivenMaxNotNumber) {
                if (isGivenMaxNotNumber && axis.min >= autoMax) {
                    axis.max = axis.min * (axis.min >= 0 ? 2 : 0.5);
                }

                if (isGivenMinNotNumber && axis.max <= autoMin) {
                    axis.min = axis.max * (axis.max >= 0 ? 0.5 : 2);
                }

                // let the minimum value of logarithmic axis stay in y=1 (sisense default since 5.0)
                if (isGivenMinNotNumber && isAxisLog) {
                    axis.min = autoMin;
                }
            }

            if (isPercent) {
                axis.min = null;
                axis.max = null;
            }

            // save the auto min and max for the styler
            axis.autoMin = autoMin;
            axis.autoMax = autoMax;
            axis.isPercent = isPercent;

        };

        // sets mask formatting to series if exist, otherwise null
        var setBoxPlotSeriesMaskedFormat = function (series, panelTitles) {

            var mask;

            mask = $mask.getMaskFunction($$get(panelTitles, 0 + '.format'));
            for (var i = 0, l = series.length; i < l; i++) {

                series[i].mask = mask;
            }

        };

        //adds outliers to the highcharts chart
        var addOutliers = function (outliers, categories, titlesForChart, defaultColor) {

            var i = 0, l = outliers.length;
            var data = [],
                categoryIndex;

           for (; i < l; i++) {

               categoryIndex = categories.indexOf($$get(outliers[i],"0.text"));
               if (categoryIndex!== -1){
                   data.push([categoryIndex,$$get(outliers[i],"1.data")])
               }
            }

            return {
                showInLegend: false,
                name: $$get(titlesForChart, 'value.0.title'),
                type: 'scatter',
                yAxis: 0,
                data: data,
                marker: {
                    fillColor: 'transparent',
                    lineWidth: 1,
                    lineColor: defaultColor
                }
            };

        };

        //sets mask formatting to axis, create the mask function only if all series have the same mask
        var setAxisMaskedFormat = function (axis, titlesForChart, axisLocation, panelOfYAxis) {

            // masking xAxis (scatter chart only)
            if (panelOfYAxis === 'xAxis') {
                axis.mask = $mask.getMaskFunction($$get(titlesForChart, panelOfYAxis + '.0.format'));
                return;
            }

            var yOry2 = axisLocation === 'right' ? 1 : 0;

            //TODO for boxplot (check is Numeric)
            var seriesJaqlMeasures = _.filter(titlesForChart[panelOfYAxis], function (ser) {
                return true; //(ser.isMeasure && ser.yAxisIndex === yOry2);
            });

            if (seriesJaqlMeasures.length === 0) {
                axis.mask = $filter('number');
                return;
            }

            // gets the first series mask
            var mask = $$get(seriesJaqlMeasures, 0 + '.format');

            // check that all series masking are the same
            for (var i = 1, l = seriesJaqlMeasures.length; i < l; i++) {

                if (JSON.stringify(mask.format.mask) !== JSON.stringify($$get(seriesJaqlMeasures, i + '.format.format.mask'))) {

                    axis.mask = $filter('number');
                    return;
                }
            }

            // create the masking function
            axis.mask = $mask.getMaskFunction(mask);

        };

        var tickPositioner = function () {
            // FIX ME: duplicates logic in genericPanel, prevent hanging up browser on switching widgets
            var MAX_MARKER_LINES_AMOUNT = 500;
            if (((this.max - this.min) / this.tickInterval) > MAX_MARKER_LINES_AMOUNT) {
                return [this.min, this.max];
            }

            if (this.min != this.dataMin) {
                var positions = [],
                    tick = this.min
                    ;

                for (tick; (tick - this.tickInterval) <= this.max; tick += this.tickInterval) {
                    positions.push(tick);
                }

                return positions;
            }
        };

        //creates an object of single yAxis
        var createYAxis = function (style, templateY, axisLocation, titlesForChart, familyType) {
            var formatWithCommas = $filter('number');

            var op = {
                type: 'linear',
                opposite: (axisLocation === 'right') ? true : false,
                title: {enabled: false, text: null, margin: 25},
                labels: {
                    formatter: function () {
                        var func = $$get(this, 'axis.options.mask');
                        return !angular.isNumber(this.value) ? this.value : (defined(func) ? func(this.value) : formatWithCommas(this.value));
                    }
                },
                tickmarkPlacement: "on",
                startOnTick: false,
                endOnTick: false
                //tickPixelInterval: 100
            };

            if (defined(templateY) && $$get(templateY, 'enabled') === true) {

                var axisTitles = self.getAxisTitles(null, familyType, titlesForChart);

                if ($$get(templateY, 'title.enabled')) {
                    op.title.enabled = true;
                    if (op.opposite) {
                        op.title.text = getTitleOrDefaultTitle($$get(templateY, 'title.text'), axisTitles.y2);
                    } else {
                        var defaultTitleTextSource = familyType === "scatter" ? 'yAxis.0.title' : "series.0.title";
                        op.title.text = getTitleOrDefaultTitle($$get(templateY, 'title.text'), axisTitles.y1);
                    }
                    op.title.style = $highchartCssStyling.generateFontStyle($$get(style, 'children.titles.style'));
                }

                if (defined(templateY.labels)) {

                    if (defined(templateY.labels.enabled)) {
                        op.labels.enabled = templateY.labels.enabled;
                    }

                    if (defined(templateY.labels.rotation)) {
                        op.labels.rotation = templateY.labels.rotation;
                    }

                    if (defined(templateY.labels.stepInterval)) {
                        op.labels.step = templateY.labels.stepInterval;
                    }

                }

                if ($$get(templateY, 'gridLines') === true) {
                    op.gridLineWidth = 1;
                    op.minorTickWidth = 0;
                    op.tickWidth = 0;
                }
                else {
                    op.gridLineWidth = 0;
                    op.gridLineColor = "transparent";

                    if (op.labels.enabled === true) {
                        op.minorTickWidth = 1;
                        op.tickWidth = 1;
                    }
                }

                // remove ticks for secondary
                if (op.opposite) {
                    op.tickWidth = (templateY.templateMainYHasGridLines ? 0 : 1);
                    op.minorTickWidth = (templateY.templateMainYHasGridLines ? 0 : 1);
                }

                if (!!templateY.isIntervalEnabled && !isNaN(parseInt(templateY.intervalJumps))) {
                    op.tickInterval = templateY.intervalJumps;
                    op.tickPositioner = tickPositioner;
                }

                // sets logarithmic axis
                if (defined(templateY.logarithmic) && templateY.logarithmic) {
                    op.type = "logarithmic";
                }

            }
            else {

                op.lineWidth = 0;
                op.minorGridLineWidth = 0;
                op.lineColor = 'transparent';
                op.gridLineColor = 'transparent';
                op.minorTickLength = 0;
                op.tickLength = 0;
                op.labels.enabled = false;

            }

            // set min and max values to the axis
            var min = parseFloat($$get(templateY, 'min')),
                max = parseFloat($$get(templateY, 'max'));

            if (isNaN(min) && isNaN(max)) {
                op.startOnTick = true;
                op.endOnTick = true;
            }
            else {
                if (isNaN(min)) {
                    //min = max / 2;
                    min = null;
                    op.startOnTick = true;
                    op.minPadding = 0;
                }
                if (isNaN(max)) {
                    //max = min * 2;
                    max = null;
                    op.endOnTick = true;
                    op.maxPadding = 0;
                }
                op.min = min;
                op.max = max;
                op.ignoreMaxPadding = true; // ignoring max and min padding both!
            }

            // fix yAxis spacing with scatter
            if (familyType === 'scatter' && $$get(titlesForChart, 'yAxis.0.isMeasure') === true) {

                op.startOnTick = (op.type === 'logarithmic' ? true : false);
                op.endOnTick = false;
            }

            // panel of yAxis
            var panelOfYAxis;
            switch (familyType) {
                case 'scatter':
                    panelOfYAxis = 'yAxis';
                    break;

                case 'plotwidget':
                    panelOfYAxis = 'value';
                    break;

                case 'cartesian':
                case 'polar':
                default:
                    panelOfYAxis = 'series';
                    break;

            }

            // sets mask formatting to axis
            setAxisMaskedFormat(op, titlesForChart, axisLocation, panelOfYAxis);

            return op;

        };

        // ignore not numbers values in series
        var ignoreNotNumbersInSeries = function (series, numericFields) {

            _.each(numericFields, function (field) {

                var val;

                for (var i = 0, l = series.length; i < l; i++) {

                    if ($$get(series, i + '.data.length') > 0) {

                        for (var j = 0, m = series[i].data.length; j < m; j++) {

                            if (series[i].data[j] != null) {

                                val = series[i].data[j][field];
                                if (defined(val) && (!angular.isNumber(val) || isNaN(val))) {
                                    series[i].data[j][field] = null;

                                }
                            }
                        }

                    }
                }
            });
        };


        // object to save in chart.events for convert symbols in legend to squares
        var setLegendSymbolsToSquares = function () {

            var events = {};

            var toSquares = function () {

                return function (event) {

                    // convert symbols in legend to squares
                    $('.highcharts-legend-item path', this.container).attr('stroke-width', '12').attr('width', '12').attr('d', 'M 0 12 L 12 12');
                    $('.highcharts-legend-item rect', this.container).attr('rx', '0').attr('ry', '0').attr('width', '12').attr('height', '12');
                };
            };

            events.load = toSquares();
            events.redraw = toSquares();

            return events;

        };

        var reverseLegendItemIfNecessary = function reverseLegendItemIfNecessary(widget, legendPosition, legendOptions) {


            // validate
            if (!angular.isString(widget.subtype) || !defined(legendPosition)) {
                return
            }

            var subtypeSplitted = widget.subtype.split('/'),
                type = $$get(subtypeSplitted, 0),
                subtype = $$get(subtypeSplitted, 1);

            if (type === 'bar') {

                var stackedRegex = new RegExp("^stacked.*"),
                    stacked100Regex = new RegExp("^stacked.*100$");

                if (subtype === 'classic' && (legendPosition === 'right' || legendPosition === 'left')) {
                    legendOptions.reversed = true;
                }

                else if ((stackedRegex.test(subtype) || stacked100Regex.test(subtype)) && (legendPosition === 'top' || legendPosition === 'bottom')) {
                    legendOptions.reversed = true;
                }
            }
        };

        // find the first active category on navigator from the given pixel location
        var findFirstCategoryFromPixel = function (series, staticHandlePos, start, end, isLast) {

            var index = Math.floor((end + start) / 2);

            if (isLast === true) {
                index++;
            }

            if (index === series.length - 1) {
                return series.length - 1;
            }

            if (index <= 1) {
                return 0;
            }

            if (series[index].clientX > staticHandlePos && series[index - 1].clientX < staticHandlePos) {
                return index;
            }

            if (series[index].clientX > staticHandlePos) {
                return findFirstCategoryFromPixel(series, staticHandlePos, start, index, false);
            }

            else {
                return findFirstCategoryFromPixel(series, staticHandlePos, index, end, (index === end - 1) ? true : false);
            }

        };

        // object to save in chart.events for navigate the scrollbar
        var setScrollerEvent = function (widget, argsForDraggingAxis) {

            var events = {};

            events.isValidResize = function (args) {

                args.chart.scroller.cancelMouseUp = false;

                return true; // disable bird's eye limitation

                if (defined(args.pxMinOld) && defined(args.pxMaxOld) && defined(args.pxMin) && defined(args.pxMax)) {

                    var minChanged = args.pxMin - args.pxMinOld, maxChanged = args.pxMax - args.pxMaxOld;
                    var navSeries = _.filter(args.chart.series, function (ser) {
                        return ser.name === 'Navigator';
                    })[0].data;
                    var staticHandlePos, startPoint, optEndPoint, optEndPointLocation;
                    var containerWidth = $(args.chart.container).width();

                    var navigatorMaximumCategories = Math.floor(containerWidth / default_chart_settings.navigatorScrollMinSpacePerItem);

                    var isOverLimit = false;

                    // validate navigator series
                    if (navSeries.length === 0 || !defined(navSeries)) {
                        return;
                    }

                    // ^ is xor. check limit only if one and only one of the edges was draged
                    if (minChanged !== 0 ^ maxChanged !== 0) {

                        if (minChanged < 0) {

                            staticHandlePos = args.pxMax;
                            startPoint = findFirstCategoryFromPixel(navSeries, staticHandlePos, 0, navSeries.length - 1, false);
                            optEndPoint = startPoint - navigatorMaximumCategories;

                            if (optEndPoint < 0) {
                                return;
                            }

                            optEndPointLocation = navSeries[optEndPoint].clientX;

                            if (optEndPointLocation > args.pxMin) {

                                isOverLimit = true;
                                args.chart.scroller.cancelMouseUp = true;
                                args.cancel = true;
                            }

                        }

                        else if (maxChanged > 0) {

                            staticHandlePos = args.pxMin;
                            startPoint = findFirstCategoryFromPixel(navSeries, staticHandlePos, 0, navSeries.length - 1, false);
                            optEndPoint = startPoint + navigatorMaximumCategories;

                            if (optEndPoint >= navSeries.length) {
                                optEndPoint = navSeries.length - 1;
                            }

                            if (optEndPoint > navSeries.length) {
                                return;
                            }

                            var pointWidth = (navSeries.length < 2) ? 0 : navSeries[1].clientX - navSeries[0].clientX;

                            optEndPointLocation = navSeries[optEndPoint].clientX + pointWidth;

                            if (optEndPointLocation < args.pxMax) {

                                isOverLimit = true;
                                args.chart.scroller.cancelMouseUp = true;
                                args.cancel = true;
                            }

                        }

                    }

                    // color the scroller in red if over limit
                    if (isOverLimit && $$get(args, 'chart.scroller.isOverLimit') !== true) {
                        $('.navigatorRightHandle', args.chart.container).attr({fill: '#FB7470', stroke: '#FB7470'});
                        $('.navigatorRightHandleSquare', args.chart.container).attr({
                            fill: '#FB7470',
                            stroke: '#FB7470'
                        });
                        $('.navigatorLeftHandle', args.chart.container).attr({fill: '#FB7470', stroke: '#FB7470'});
                        $('.navigatorLeftHandleSquare', args.chart.container).attr({
                            fill: '#FB7470',
                            stroke: '#FB7470'
                        });
                        $$set(args, 'chart.scroller.isOverLimit', true);
                    }
                    else if (!isOverLimit && $$get(args, 'chart.scroller.isOverLimit') === true) {
                        $('.navigatorRightHandle', args.chart.container).attr({fill: '#D4D4D4', stroke: '#D4D4D4'});
                        $('.navigatorRightHandleSquare', args.chart.container).attr({
                            fill: '#D4D4D4',
                            stroke: '#D4D4D4'
                        });
                        $('.navigatorLeftHandle', args.chart.container).attr({fill: '#D4D4D4', stroke: '#D4D4D4'});
                        $('.navigatorLeftHandleSquare', args.chart.container).attr({
                            fill: '#D4D4D4',
                            stroke: '#D4D4D4'
                        });
                        $$set(args, 'chart.scroller.isOverLimit', false);
                    }

                    // place labels of xAxis horizonal or vertical due the container width
                    var mainXaxis = $$get(args, 'chart.xAxis.' + 0);
                    if ($$get(args, 'chart.options.chart.type') !== 'bar' && $$get(mainXaxis, 'options.labels.enabled') === true && $$get(document, 'body.style.cursor') !== 'move') {

                        var rotationStatus = $$get(mainXaxis, 'rotationStatus');
                        if (!defined(rotationStatus)) {
                            rotationStatus = $$get(mainXaxis, 'options.labels.rotation') || 0;
                        }

                        if ($$get(document, 'body.style.cursor') !== 'move') {

                            // convert from horizonal to vertical
                            if (rotationStatus === 0) {
                                var firstLabel = findFirstCategoryFromPixel(navSeries, args.pxMin, 0, navSeries.length - 1, false),
                                    lastLabel = findFirstCategoryFromPixel(navSeries, args.pxMax, 0, navSeries.length - 1, false),
                                    numLabelsInWindow = lastLabel - firstLabel;

                                if (containerWidth < numLabelsInWindow * default_chart_settings.minSpacePerItemHorizonalLabelXAxis) {
                                    mainXaxis.rotationStatus = 90;
                                    args.chart.xAxis[0].update({
                                        min: firstLabel,
                                        max: lastLabel,
                                        labels: {rotation: 90, align: 'left', x: -4, y: 20}
                                    });
                                }

                            }

                            // convert from vertical to horizonal
                            if (rotationStatus === 90) {
                                var firstLabel = findFirstCategoryFromPixel(navSeries, args.pxMin, 0, navSeries.length - 1, false),
                                    lastLabel = findFirstCategoryFromPixel(navSeries, args.pxMax, 0, navSeries.length - 1, false),
                                    numLabelsInWindow = lastLabel - firstLabel;

                                if (containerWidth > numLabelsInWindow * default_chart_settings.minSpacePerItemHorizonalLabelXAxis) {
                                    mainXaxis.rotationStatus = 0;
                                    args.chart.xAxis[0].update({
                                        min: firstLabel,
                                        max: lastLabel,
                                        labels: {rotation: 0, align: 'center', x: 0, y: 20}
                                    });
                                }
                            }

                        }

                    }

                }

            };

            events.storeScrollerLocation = function (args) {

                if (widget.options.previousScrollerLocation || args.ext) {

                    $$set(widget, 'options.previousScrollerLocation', $$get(args, 'ext'));
                    widget.changesMade('scroll');
                }

                // attach dragging xAxis to control length of vertical labels if needed
                $draggingAxis.dragAxis(argsForDraggingAxis.$lmnt, args.chart, $$get(args.chart, 'xAxis.0'), 'navigatorMoved', argsForDraggingAxis.scope);
            };

            return events;

        };

        this.setXAxisDragEvent = function setXAxisDragEvent(widget) {
            var events = {},
                that = this;

            events.xAxisDrag = function xAxisDrag(scope, $lmnt, args) {
                that.renderInternally(scope, $lmnt, args);
            };

            events.storeMaxCharsPerLabelXAxis = function storeMaxCharsPerLabelXAxis(args) {

                if (defined(args.maxCharsPerLabelXAxis)) {

                    $$set(widget, 'options.maxCharsPerLabelXAxis', $$get(args, 'maxCharsPerLabelXAxis'));
                    widget.changesMade('xAxisDrag');
                }
            };

            events.fixResizeableDivPosition = function fixResizeableDivPosition(chart) {
                _.defer(function () {
                    $draggingAxis.fixResizeableDivPosition(chart);
                });
            };

            return events;
        };

        this.renderInternally = function renderInternally(scope, $lmnt, args) {
            this.render(scope, $lmnt, args);
        }

        // draw errors panel box
        var drawErrorsBox = function (chart, errors) {

            var oldErrorBox = $('.highcharts-errors-box', chart.renderTo)[0];
            if (defined(oldErrorBox)) {
                oldErrorBox.remove();
            }

            chart.errorBox = chart.renderer.g('highcharts-errors-box')
                .attr({zIndex: 10})
                .add();

            var svgNameSpace = 'http://www.w3.org/2000/svg';
            var fontHeight = 12, maxTextWidth = 0, currentTextWidth, marginTop = 30;

            // creates the box that will hold the error texts
            var box2 = document.createElementNS(svgNameSpace, 'g');
            box2.setAttribute('class', 'error-box-panel');
            box2.setAttribute('zIndex', 20);
            chart.errorBox.element.appendChild(box2);

            _.each(errors, function (er, i) {

                var errorText = getErrorMessage(er);

                if (angular.isObject(er)) {
                    errorText = getErrorMessage(er.errorKey);
                    /** SisenseJS.String.format */
                    errorText = window.sisenseUtils.string.format(errorText, er.args[0], er.args[1], er.args[2]);
                }

                var text = document.createElementNS(svgNameSpace, 'text');
                text.setAttribute("x", chart.chartWidth / 2);
                text.setAttribute("y", i * fontHeight + marginTop);
                text.setAttribute("text-anchor", "middle");
                text.setAttribute("transform", "translate(0,0)");
                text.setAttribute("opacity", 1);
                text.setAttribute("fill", 'white');
                //text.setAttribute("style", "font-family: opensansregular;font-size: 13px;cursor: default;line-height: 14px;width: 49px;height:" + fontHeight + "px");
                text.setAttribute("style", "font:" + fontHeight + "px ubuntulight, sans-serif;");
                text.textContent = errorText;

                box2.appendChild(text);

                currentTextWidth = text.getBBox().width;
                if (currentTextWidth > maxTextWidth) {
                    maxTextWidth = currentTextWidth
                }
            });

            var marginHeight = 12, marginWidth = 40;
            var rect = document.createElementNS(svgNameSpace, 'rect');
            rect.setAttribute("x", (chart.chartWidth / 2) - (maxTextWidth / 2) - (marginWidth / 2));
            rect.setAttribute("y", 15); // marginTop / 2);
            rect.setAttribute("rx", 3);
            rect.setAttribute("ry", 3);
            rect.setAttribute("width", maxTextWidth + marginWidth);
            rect.setAttribute("height", errors.length * fontHeight + marginHeight);
            rect.setAttribute("opacity", 0.8);
            rect.setAttribute("fill", "#666666");
            rect.setAttribute("transform", "translate(0,0)");

            box2.insertBefore(rect, box2.firstChild);

        };

        // disable chart animations
        var setAnimationProperty = function (series, disableAnimations) {

            if (disableAnimations) {
                _.each(series, function (ser) {
                    ser.animation = false;
                });
            }
        };

        // add slection events to the chart
        var addSelectionEvents = function (widget, $scope, chartOptions, $lmnt) {

            var appstate = widget.appstate || $rootScope.appstate,
                familyType = $$get(chartOptions, '$familyType'),
                drillOnly = false,
                leftClickEnable = true;


            // disable selection & drill for mobiles #9466 - consumer actions not should be enabled on mobile
            if ($device.phone()) {
                return;
            }


            // disable selection in widget edit mode or if on tablet platform
            if (appstate === 'widget') {
                drillOnly = true;
            }
            if (widget.disableSelctor || appstate === 'widget') {
                leftClickEnable = false;
            }

            $scope.selectionSession;

            $scope.pointsStatusBefore;

            $scope.selectionSessionStarted = function (chartFamilyType, drillOnly, isCtrlDown) {

                var ctrlKeyUp = function (event) {

                    var replace = !isCtrlDown;

                    $highchartSelections.closeSession($scope, chartFamilyType, event, replace, false, drillOnly);

                    $scope.selectionSession = null;
                    $scope.pointsStatusBefore = null;

                    $(document).off("keyup", ctrlKeyUp);
                };

                $(document).on("keyup", ctrlKeyUp);

            };

            var placeToEvent;

            switch (familyType) {

                case 'cartesian':
                case 'polar':

                    // click event
                    placeToEvent = $$setNget(chartOptions, 'plotOptions.series.point.events');
                    if (leftClickEnable) {
                        placeToEvent.click = $highchartSelections.setCartesianSelectionEvent(widget, $scope, true, drillOnly);
                    }
                    placeToEvent.contextmenu = $highchartSelections.setCartesianSelectionEvent(widget, $scope, false, drillOnly);

                    // drag event
                    $$set(chartOptions, 'chart.zoomType', 'x');
                    placeToEvent = $$setNget(chartOptions, 'chart.events');
                    placeToEvent.selection = $highchartSelections.setCartesianSelectionEvent(widget, $scope, false, drillOnly);

                    break;

                case 'pie':

                    // click event
                    placeToEvent = $$setNget(chartOptions, 'plotOptions.pie.point.events');
                    if (leftClickEnable) {
                        placeToEvent.click = $highchartSelections.setPieSelectionClickEvent(widget, $scope, true, drillOnly);
                    }
                    placeToEvent.contextmenu = $highchartSelections.setPieSelectionClickEvent(widget, $scope, false, drillOnly);

                    break;

                case 'scatter':

                    // click event
                    placeToEvent = $$setNget(chartOptions, 'plotOptions.series.point.events');
                    if (leftClickEnable) {
                        placeToEvent.click = $highchartSelections.setScatterSelectionEvent(widget, $scope, true, drillOnly);
                    }
                    placeToEvent.contextmenu = $highchartSelections.setScatterSelectionEvent(widget, $scope, false, drillOnly);

                    // drag event
                    $$set(chartOptions, 'chart.zoomType', 'xy');
                    placeToEvent = $$setNget(chartOptions, 'chart.events');
                    placeToEvent.selection = $highchartSelections.setScatterSelectionEvent(widget, $scope, false, drillOnly);

                    break;

                case 'plotwidget':


                    // click event
                    placeToEvent = $$setNget(chartOptions, 'plotOptions.series.point.events');
                    if (leftClickEnable) {
                        placeToEvent.click = $highchartSelections.setCartesianSelectionEvent(widget, $scope, true, drillOnly);
                    }
                    placeToEvent.contextmenu = $highchartSelections.setCartesianSelectionEvent(widget, $scope, false, drillOnly);

                    // drag event
                    $$set(chartOptions, 'chart.zoomType', 'x');
                    placeToEvent = $$setNget(chartOptions, 'chart.events');
                    placeToEvent.selection = $highchartSelections.setCartesianSelectionEvent(widget, $scope, false, drillOnly);

                    break;

            }

        };

        var getErrorMessage = function (errorKey) {
            var $translate = $filter('translate');

            return $translate(errorKey);
        }

        //
        // PUBLICS

        // gets the basic common highcharts options object
        this.options = function (familyType, titlesForChart) {

            return {

                chart: {},
                legend: {},
                tooltip: {},
                scrollbar: {enabled: false},
                title: {text: null},
                subtitle: {text: null},
                credits: {enabled: false},
                loading: null,
                navigation: null,
                exporting: {enabled: false},
                errors: [],

                $familyType: familyType,
                $filtersAffected: {
                    x: $$get(titlesForChart, 'affectFromGlobalFilterX')
                },

                $dataForAutomatedTests: {
                    // first term for scatter, second for all cartesian, third is default
                    isDimColor: ($$get(titlesForChart, 'color.0.isMeasure') === false) || defined(_.find(titlesForChart.series, function (ser) {
                        return ser.isMeasure === false
                    })) || false,
                    isDimXAxis: defined(_.find(titlesForChart.xAxis, function (item) {
                        return item.isMeasure === false
                    })) || false,
                    isDimYAxis: defined(_.find(titlesForChart.yAxis, function (item) {
                        return item.isMeasure === false
                    })) || false

                }
            }
        };

        // generate tooltip options object from the given style/ template
        this.tooltip = function (widget, titlesForChart, queryResult, style, template, fullType, plotOptions, series, familyType) {

            // decide what crosshairs to render
            var type = fullType.split('/')[0], showXCrosshair = false, showYCrosshair = false;
            if (type) {

                showXCrosshair = type !== 'bar' && type !== 'column' && type !== 'pie' && type !== 'bubble' && type !== 'boxplot';
                showYCrosshair = false;//type === 'bubble';
            }

            var op = {
                crosshairs: [showXCrosshair, showYCrosshair]
            };

            switch (familyType) {

                case 'pie':
                    $.extend(plotOptions.pie.point.events, {mouseOver: $highchartTooltip.generateTooltip(widget, titlesForChart, queryResult, familyType, $$get(default_chart_settings, 'tooltipSettings.pie'), series)});
                    break;

                case 'cartesian':
                case 'polar':

                    op.shared = default_chart_settings.tooltipSettings.cartesian.cartesianShowMultiSeries < series.length ? true : false;

                    if (!defined($$get(plotOptions, 'series.point.events'))) {
                        $$set(plotOptions, 'series.point.events', {});
                    }

                    $.extend(plotOptions.series.point.events, {mouseOver: $highchartTooltip.generateTooltip(widget, titlesForChart, queryResult, familyType, $$get(default_chart_settings, 'tooltipSettings.cartesian'), series)});
                    break;

                case 'scatter':

                    if (!defined($$get(plotOptions, 'series.point.events'))) {
                        $$set(plotOptions, 'series.point.events', {});
                    }

                    $.extend(plotOptions.series.point.events, {mouseOver: $highchartTooltip.generateTooltip(widget, titlesForChart, queryResult, familyType, $$get(default_chart_settings, 'tooltipSettings.scatter'), series)});
                    break;
                case 'plotwidget':
                    if (!defined($$get(plotOptions, 'boxplot.point.events'))) {
                        $$set(plotOptions, 'boxplot.point.events', {});
                        $$set(plotOptions, 'scatter.point.events', {}); // set event for Outliers
                    }

                    $.extend(plotOptions.boxplot.point.events, {mouseOver: $plotHighchartTooltip.generateTooltip(widget, titlesForChart, queryResult, familyType, $$get(default_chart_settings, 'tooltipSettings.scatter'), series)});
                    $.extend(plotOptions.scatter.point.events, {mouseOver: $plotHighchartTooltip.generateTooltipOutliers(widget, titlesForChart, queryResult, familyType, $$get(default_chart_settings, 'tooltipSettings.scatter'), series)});

                    break;
            }

            /*
             // generate fill style
             $.extend(op, $highchartCssStyling.generateFillStyle(style));

             // generate border style
             $.extend(op, $highchartCssStyling.generateBorderStyle(style));

             // generate font style
             op.style = $highchartCssStyling.generateFontStyle(style);
             */
            op.enabled = false;

            return op;
        };

        // generate navigator - bird's eye for cartesian charts
        this.navigator = function (widget, style, chartType, series, xAxisTemplate) {

            var op = {
                enabled: false,
                margin: 30,
                height: 40, // will be override in highchart.3.0.0
                baseSeries: 0, // may be override with the longest series
                xAxis: {labels: {enabled: false}},
                series: {
                    type: chartType,
                    color: $$get(series, 0 + '.color'),
                    dataGrouping: {smoothed: false}
                },
                handles: {
                    backgroundColor: '#CCCCCC',
                    borderColor: '#CCCCCC'
                },
                maskFill: 'rgba(255, 255, 255, 0.75)',
                outlineColor: '#CCCCCC',
                handlesColor: '#D4D4D4',
                handlesRiFlesColor: '#FFF'
            };

            if (true || $$get(widget, 'appstate') !== 'newwidget' && chartType !== 'bar' && $$get(xAxisTemplate, 'enabled') === true && $$get(xAxisTemplate, 'labels.enabled') === true) {

                if ($$get(style, 'enabled') === true) {
                    op.enabled = true;
                }
            }

            // for going back from enabled=false due to appstate=newwidget
            $$set(op, 'enabledFromEditor', $$get(style, 'enabled'));

            var reportingConf = reportingConfig.getConfig();
            if ($$get(reportingConf, "widgets.basicchart.showBirdsEye") === false) {
                op.enabled = false;
                op.enabledFromEditor = false;
            }

            return op;
        };


        // generate legend object from the given style/ template
        this.legend = function (widget, titlesForChart, familyType, style, template) {

            var op = {
                itemStyle: {cursor: 'default'}
            };

            // generate fill style
            $.extend(op, $highchartCssStyling.generateFillStyle(style));

            // generate border style
            $.extend(op, $highchartCssStyling.generateBorderStyle(style));

            // generate font style
            $.extend(op.itemStyle, $highchartCssStyling.generateFontStyle(style));

            if (defined(template.enabled)) {
                op.enabled = template.enabled && $$get(widget, 'appstate') !== 'newwidget';
            }

            // for going back from enabled=false due to appstate=newwidget
            $$set(op, 'enabledFromEditor', $$get(template, 'enabled'));

            var chartType = $$get(widget.type.split('/'), '1');

            if (defined(template.position)) {
                switch (template.position) {
                    case 'top':
                        op.align = 'center';
                        op.verticalAlign = 'top';
                        op.layout = 'horizontal';

                        // set legend above the grouping xAxis labels if exists
                        if (familyType === 'cartesian' && chartType !== 'bar' && $$get(titlesForChart, 'xAxis.length') > 1) {
                            op.legendAboveGrouping = true;
                        }

                        break;
                    case 'right':
                        op.align = 'right';
                        op.verticalAlign = 'middle';
                        op.layout = 'vertical';

                        // set legend above the grouping xAxis labels if exists
                        if (familyType === 'cartesian' && chartType === 'bar' && $$get(titlesForChart, 'xAxis.length') > 1) {
                            op.legendRightToGrouping = true;
                        }

                        break;
                    case 'bottom':
                        op.align = 'center';
                        op.verticalAlign = 'bottom';
                        op.layout = 'horizontal';
                        break;
                    case 'left':
                        op.align = 'left';
                        op.verticalAlign = 'middle';
                        op.layout = 'vertical';
                        break;
                }
            }

            reverseLegendItemIfNecessary(widget, $$get(template, 'position'), op);

            return op;
        };

        // generate chart object from the given style/ template
        this.chart = function (style, template, fullType, familyType, widget, titleForChart) {

            var splittedType = fullType.split('/'),
                chartType = splittedType[0],
                subType = splittedType[1];

            var spacingTop = 20, spacingRight = 20;

            if (familyType === 'cartesian' && $$get(titleForChart, 'xAxis.length') > 1) {
                if (chartType !== 'bar') {
                    spacingTop = 45;
                }
                else {
                    spacingRight = 90;
                }
            }

            var op = {spacing: [spacingTop, spacingRight, 20, 20], alignTicks: false};

            // generate fill style
            $.extend(op, $highchartCssStyling.generateFillStyle(style));

            // generate border style
            $.extend(op, $highchartCssStyling.generateBorderStyle(style));

            // chart type
            var splineRegex = new RegExp(".*spline.*");

            // suppose to be true always
            if (defined(subType)) {

                // check for spline mode type
                if (splineRegex.test(subType)) {

                    switch (splittedType[0]) {
                        case 'area':
                            op.type = 'areaspline';
                            break;
                        case 'line':
                            op.type = 'spline';
                            break;
                    }
                }

                // not spline type
                else {
                    op.type = splittedType[0];
                }

            }
            // never suppose to happen but, in case, the default is line chart
            else {
                op.type = 'line';
            }

            var chartEvents = $$setNget(op, 'events');

            // convert symbols in legend to squares
            $.extend(chartEvents, setLegendSymbolsToSquares());

            return op;
        };


        /** Titles Section */
        this.getAxisTitles = function (widget, familyType, titlesForChart) {
            var titles = titlesForChart || self.findTitles(widget, familyType, null);

            var x1 = familyType === "plotwidget" ? $$get(titles, "category.0.title") : $$get(titles, "xAxis.0.title");

            var titlesTexts = {
                "x1": x1,
                "x2": $$get(titles, "xAxis.1.title")
            };
            var y1 = [];
            var y2 = [];
            var y2Prop = "series";

            switch (familyType) {

                case "scatter":
                    y2Prop = "yAxis";
                    break;

                case "plotwidget":
                    y2Prop = "value";
                    break;
            }

            _.each(titles[y2Prop], function (item) {
                if (item.yAxisIndex === 1) {
                    y2.push(item.title)
                } else {
                    y1.push(item.title)
                }
            });
            titlesTexts.y1 = y1.length === 0 ? undefined : y1.length === 1 ? y1[0] : "";
            titlesTexts.y2 = y2.length === 0 ? undefined : y2.length === 1 ? y2[0] : "";

            return titlesTexts;
        }


        function getTitleOrDefaultTitle(title, defaultTitle) {
            var resultTitle = (title === null || title === undefined) ? defaultTitle : title === "" ? " " : title;
            return resultTitle;
        }

        function generateAxisTopLabel(axisTitleTopLabel, text, style) {
            axisTitleTopLabel.html = text;
            axisTitleTopLabel.style = style;
            axisTitleTopLabel.labelType = 'axisTitle';
            return axisTitleTopLabel;
        }

        function addAxisTitleOnBottom(op, title, style) {
            op.title.enabled = true;
            op.title.text = title;
            op.title.style = style;
        }

        this.addAxisTitleOnTop = function (options, title) {

            if (!_.isEmpty(title)) {
                if (!_.isArray($$get(options, 'labels.items'))) {
                    $$set(options, 'labels.items', []);
                }
                options.labels.items.push(title);
            }
        }


        function addXAxesTitles(op, style, template, familyType, titlesForChart, axisTitleTopLabel) {
            var axisTitles = self.getAxisTitles(null, familyType, titlesForChart);


            var x1Title = getTitleOrDefaultTitle($$get(template, 'title.text'), axisTitles.x1);
            var x2Title = getTitleOrDefaultTitle($$get(template, 'x2Title.text'), axisTitles.x2);

            if (defined(x2Title)) {
                if ($$get(template, 'x2Title.enabled')) {
                    addAxisTitleOnBottom(op, x2Title, style);
                }

                if (defined(x1Title) && $$get(template, 'title.enabled')) {
                    generateAxisTopLabel(axisTitleTopLabel, x1Title, style);
                }

            } else if (defined(x1Title) && $$get(template, 'title.enabled')) {
                if (familyType === "polar") {
                    generateAxisTopLabel(axisTitleTopLabel, x1Title, style);
                } else {
                    addAxisTitleOnBottom(op, x1Title, style);
                }
            }
        }

        /** END of titles Section */

            // generate xaxis legend object from the given style/ template
        this.xaxis = function (widget, style, template, titlesForChart, axisTitleTopLabel, familyType) {

            var formatWithCommas = $filter('number');

            var op = {

                labels: {
                    staggerLines: 1,
                    overflow: 'none',
                    rotation: 0,
                    maxCharactersPerLabel: $$get(widget, 'options.maxCharsPerLabelXAxis') || default_chart_settings.maxCharsPerLabelXAxis,
                    formatter: function () {
                        var func = $$get(this, 'axis.options.mask');
                        return !angular.isNumber(this.value) ? this.value : (defined(func) ? func(this.value) : formatWithCommas(this.value));
                    }
                },
                categories: [],
                tickmarkPlacement: "on",
                tickInterval: null,
                minorTickInterval: null,
                isDate: _.some($$get(titlesForChart, 'xAxis'), function (axis) {
                    return axis.isDate;
                }),
                title: {enabled: false, margin: 25}
                // tickPixelInterval: null
            };

            if (defined(template) && defined(template.enabled) && template.enabled) {

                var titlesStyle = $highchartCssStyling.generateFontStyle($$get(style, 'children.titles.style'));
                addXAxesTitles(op, titlesStyle, template, familyType, titlesForChart, axisTitleTopLabel);

                if (defined(template.labels)) {

                    if (defined(template.labels.enabled)) {
                        op.labels.enabled = template.labels.enabled;
                    }
                }

                if ($$get(template, 'gridLines') === true) {
                    op.gridLineWidth = 1;
                    op.minorTickLength = 0;
                    op.tickLength = 0;
                }
                else {
                    op.gridLineColor = "transparent";

                    if (op.labels.enabled === true) {
                        op.minorTickLength = 2;
                        op.tickLength = 5;
                    }
                }

                // sets logarithmic axis
                if (defined(template.logarithmic) && template.logarithmic) {
                    op.type = "logarithmic";
                }

                if (!!template.isIntervalEnabled && !isNaN(parseInt(template.intervalJumps))) {
                    op.tickInterval = template.intervalJumps;
                    op.tickPositioner = tickPositioner;
                }

                // set min and max values to the axis
                var min = parseFloat($$get(template, 'min')),
                    max = parseFloat($$get(template, 'max'));

                // set min max to a measure axis
                if (isNaN(min) && isNaN(max)) {
                    op.startOnTick = true;
                    op.endOnTick = true;
                }
                else {
                    if (isNaN(min)) {
                        //min = max / 2;
                        min = null;
                        op.startOnTick = true;
                        op.minPadding = 0;
                    }
                    if (isNaN(max)) {
                        //max = min * 2;
                        max = null;
                        op.endOnTick = true;
                        op.maxPadding = 0;
                    }
                    op.min = min;
                    op.max = max;
                    op.ignoreMaxPadding = true; // ignoring max and min padding both!
                }

            }
            else {

                op.lineWidth = 0;
                op.gridLineWidth = 0;
                op.minorGridLineWidth = 0;
                op.lineColor = 'transparent';
                op.minorTickLength = 0;
                op.tickLength = 0;
                op.labels.enabled = false;
            }

            // set scatter specials for xAxis
            if (familyType === 'scatter') {
                op.startOnTick = false;
                op.endOnTick = false;

                // set mask to scatter xAxis measure
                if ($$get(titlesForChart, 'xAxis.0.isMeasure') === true) {
                    setAxisMaskedFormat(op, titlesForChart, null, 'xAxis');
                }
            }
            return op;
        };

        //generate yaxis (Major and Minors) object from the given style/ template
        this.yaxis = function (style, templateY, templateY2, titlesForChart, familyType) {

            var yAxis = [];

            var isExistLeftYAxis = defined(_.findWhere(($$get(titlesForChart, 'series') || $$get(titlesForChart, 'yAxis') || $$get(titlesForChart, 'value') ), {yAxisIndex: 0})),
                isExistRightYAxis = defined(_.findWhere(($$get(titlesForChart, 'series') || $$get(titlesForChart, 'yAxis') || $$get(titlesForChart, 'value')), {yAxisIndex: 1}));

            // creates left yAxis if needed
            if (isExistLeftYAxis) {
                yAxis.push(createYAxis(style, (!templateY.inactive ? templateY : (!templateY2.inactive ? templateY2 : templateY)), 'left', titlesForChart, familyType));
            }

            // creates right yAxis if needed
            if (isExistRightYAxis) {
                templateY2.templateMainYHasGridLines = templateY.gridLines;
                yAxis.push(createYAxis(style, templateY2, 'right', titlesForChart, familyType));
            }

            if (!isExistLeftYAxis && !isExistRightYAxis) {
                yAxis.push(createYAxis(style, {
                    enabled: true,
                    labels: {enabled: true}
                }, 'left', titlesForChart, familyType));
            }

            return yAxis;

        };

        //find titles for axis and series of the requested jaql
        this.findTitles = function (widget, familyType, queryResultMetadata) {

            var metadata = widget.metadata,
                dimsForSelected,
                isY2Possible = true,
                dashboardFilterMode = $$get(widget, 'options.dashboardFiltersMode');

            switch (familyType) {

                case 'scatter':
                    dimsForSelected = [0, 1, 2, 3];
                    isY2Possible = false;
                    break;

                case 'cartesian':
                    dimsForSelected = [0];
                    break;

                case 'plotwidget':
                    dimsForSelected = [0, 1];
                    break;

                case 'polar':
                case 'pie':
                default:
                    dimsForSelected = [0];
                    isY2Possible = false;
                    break;

            }
            ;

            var titles = {},
                indexesUsed = [];

            _.each(metadata.panels, function (panel) {
                titles[$metadataManipulate.getPanelMappingName(panel.name)] = [];
            });

            _.each(metadata.panels, function (panel, index) {

                _.each(panel.getEnabledItems(), function (item) {

                    var itemToPush = {},
                        isMeasure = $jaql.isMeasure(item.jaql),
                        isDate = $jaql.isDateTimeDimension(item.jaql),
                        isNumericDimension = $jaql.isNumericDimension(item.jaql);

                    itemToPush.title = item.jaql.title || item.jaql.dim;

                    itemToPush.isMeasure = isMeasure;
                    itemToPush.isDate = isDate;
                    itemToPush.isNumericDimension = isNumericDimension;

                    itemToPush.format = item;

                    if (index === 1) {

                        itemToPush.yAxisIndex = ($$get(item, 'y2') === true && isY2Possible) ? 1 : 0;

                        var singleSeriesType = $$get(item, 'singleSeriesType');

                        if (defined(singleSeriesType)) {

                            singleSeriesType = singleSeriesType.replace(' ', '').toLowerCase();
                            itemToPush.singleSeriesType = singleSeriesType;
                        }

                    }

                    if (dashboardFilterMode !== 'filter') {

                        if (!isMeasure && !metadata.$$widget.metadata.ignore.all && metadata.$$widget.dashboard.filters.item(item.jaql, true) && !_.contains(metadata.$$widget.metadata.ignore.dimensions, item.jaql.dim) && _.contains(dimsForSelected, index)) {
                            titles['affectFromGlobalFilter' + (index === 0 ? 'X' : (index === 1 ? 'Y' : (index === 2 ? 'Point' : 'Color')))] = true;

                            var filterItem = metadata.$$widget.dashboard.filters.item(true, item.jaql).jaql.filter;

                            if (filterItem.all === true || $$get(filterItem, 'members.length') === 0) {
                                //titles['filterAll' + (index === 0 ? 'X' : (index === 1 ? 'Y' : (index === 2 ? 'Point' : 'Color')))] = true;
                                itemToPush.filterAll = true;
                                itemToPush.affectFromGlobalFilter = false;
                            }
                            else {
                                itemToPush.affectFromGlobalFilter = true;
                            }
                        }
                    }

                    // give indexing to item according the item location in the jaql metadata
                    if (queryResultMetadata) {
                        for (var i = 0, l = queryResultMetadata.length; i < l; i++) {
                            if ($$.object.areEqual($$.object.plain(item.jaql), $$.object.plain(queryResultMetadata[i].jaql)) && !_.contains(indexesUsed, queryResultMetadata[i].index)) {
                                itemToPush.indexInJaqlMetadata = i;
                                indexesUsed.push(i);
                                break;
                            }
                        }
                    }

                    titles[$metadataManipulate.getPanelMappingName(panel.name)].push(itemToPush);

                });

            });

            return titles;

        };

        // adds css style for the chart axes
        this.addCssStyleForAxes = function (style, options, axisBinary) {

            var elementToStyle, elementStyling;

            // no style for axes in css
            if (!defined(style)) {
                return;
            }

            var indexYAxisLeft = 0,  // always will be the first yAxis in the array if exists
                indexYAxisRight = $$get(options, 'yAxis.length') < 2 ? 0 : 1; // will be the first in the array if yAxis left is not active, second otherwise

            // 'widget axes' -> selects both axes for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'style');
            if (defined(elementStyling)) {

                // 1 - x-Axis enabled
                if ((axisBinary & 1) === 1) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateAxesLinesWithBorderProperties(elementStyling));
                }

                // 16 - y-Axis enabled
                if ((axisBinary & 16) === 16) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateAxesLinesWithBorderProperties(elementStyling));
                }

                // 512 - y-Axis enabled
                if ((axisBinary & 512) === 512) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateAxesLinesWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes ticks.major' -> selects only major ticks on both axes for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'children.ticks.states.major.style');
            if (defined(elementStyling)) {

                // 3 = 1 + 2 = x-Axis enabled & x ticks enabled
                if ((axisBinary & 3) === 3) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorTicksWithBorderProperties(elementStyling));
                }

                // 48 = 16 + 32 = y-Axis enabled & y ticks enabled
                if ((axisBinary & 48) === 48) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorTicksWithBorderProperties(elementStyling));
                }

                // 1536 = 512 + 1024 = y-Axis enabled & y ticks enabled
                if ((axisBinary & 1536) === 1536) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorTicksWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes ticks.minor' -> selects only minor ticks on both axes for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'children.ticks.states.minor.style');
            if (defined(elementStyling)) {

                // 3 = 1 + 2 = x-Axis enabled & x ticks enabled
                if ((axisBinary & 3) === 3) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorTicksWithBorderProperties(elementStyling));
                }

                // 48 = 16 + 32 = y-Axis enabled & y ticks enabled
                if ((axisBinary & 48) === 48) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorTicksWithBorderProperties(elementStyling));
                }

                // 1536 = 512 + 1024 = y-Axis enabled & y ticks enabled
                if ((axisBinary & 1536) === 1536) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorTicksWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes gridlines' -> selects both axes gridlines for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'children.gridlines.style');
            if (defined(elementStyling)) {

                // 9 = 1 + 8 = x-Axis enabled & x gridLines enabled
                if ((axisBinary & 9) === 9) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));

                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }

                // 144 = 16 + 128 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 144) === 144) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));

                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }

                // 4608 = 512 + 4096 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 4608) === 4608) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));

                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes grid.major' -> selects only major grid lines on both axes for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'children.grid.states.major.style');
            if (defined(elementStyling)) {

                // 9 = 1 + 9 = x-Axis enabled & x gridLines enabled
                if ((axisBinary & 9) === 9) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));
                }

                // 144 = 16 + 128 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 144) === 144) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));
                }

                // 4608 = 512 + 4096 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 4608) === 4608) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMajorGridLinesWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes grid.minor' -> selects only minor grid lines on both axes for all cartesian charts, supports all border properties
            elementStyling = $$get(style, 'children.grid.states.minor.style');
            if (defined(elementStyling)) {

                // 9 = 1 + 9 = x-Axis enabled & x gridLines enabled
                if ((axisBinary & 9) === 9) {
                    elementToStyle = $$setNget(options, 'xAxis');
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }

                // 144 = 16 + 128 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 144) === 144) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }

                // 4608 = 512 + 4096 = y-Axis enabled & y gridLines enabled
                if ((axisBinary & 4608) === 4608) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight);
                    $.extend(elementToStyle, $highchartCssStyling.generateMinorGridLinesWithBorderProperties(elementStyling));
                }
            }

            // 'widget axes labels' -> selects both axes labels for all cartesian charts, supports all text properties
            elementStyling = $$get(style, 'children.labels.style');
            if (defined(elementStyling)) {

                // 5 = 1 + 4 = x-Axis enabled & x labels enabled
                if ((axisBinary & 5) === 5) {
                    elementToStyle = $$setNget(options, 'xAxis.labels.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }

                // 82 = 16 + 64 = y-Axis enabled & y labels enabled
                if ((axisBinary & 82) === 82) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft + '.labels.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }

                // 2560 = 512 + 2048 = y-Axis enabled & y labels enabled
                if ((axisBinary & 2560) === 2560) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight + '.labels.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }
            }

            // 'widget axes title' -> selects both axes gridlines for all cartesian charts, supports all text properties
            elementStyling = $$get(style, 'children.title.style');
            if (defined(elementStyling)) {

                // 1 - x-Axis enabled
                if ((axisBinary & 1) === 1) {
                    elementToStyle = $$setNget(options, 'xAxis.title.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }

                // 16 - y-Axis enabled
                if ((axisBinary & 16) === 16) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisLeft + '.title.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }

                // 512 - y-Axis enabled
                if ((axisBinary & 512) === 512) {
                    elementToStyle = $$setNget(options, 'yAxis.' + indexYAxisRight + '.title.style');
                    $.extend(elementToStyle, $highchartCssStyling.generateFontStyle(elementStyling));
                }
            }

        };

        this.boxplotSeriesAndCategories = function (titlesForChart, queryResult, xAxis, plotOptions, deafultType, navigator, dataLimits, errors, widget, enabledOutliers, outliers) {

            var defaultColor = $$get(widget.dashboard.style.palette(), 0);
            var series = [];
            if (queryResult.error || queryResult.$$rows.length === 0) {
                return series;
            }

            var hasDummySeries = false;
            var notHaveColorsYet = true;
            var subtypeSplitted = widget.subtype.split('/'),
                type = $$get(subtypeSplitted, 0),
                subtype = $$get(subtypeSplitted, 1);

            titlesForChart.boxFillColor = (subtype === 'hollow') ? 'transparent' : defaultColor;

            //create colors for boxplot divider and lower quartile color
            var f = 0.15;
            titlesForChart.lowerQuartileColor = (subtype === 'hollow') ? 'transparent' : new Color(defaultColor).scaleBrightness(-1 * f).toHex();
            titlesForChart.boxDividerColor = new Color(defaultColor).scaleBrightness(-2 * f).toHex();

            // add dummy value as series to queryResult
            if (!$$get(titlesForChart, "value.0.isNumericDimension")) { //titlesForChart.value.length === 0

                titlesForChart.value.splice(0, 1);
                titlesForChart.value.push({isNumericDimension: true, title: '$dummySeries'});
                _.each(queryResult.$$rows, function (row) {
                    row.push({data: null, text: "0"}); // q1
                    row.push({data: null, text: "0"}); // median
                    row.push({data: null, text: "0"}); // q3
                    row.push({data: null, text: "0"}); // low
                    row.push({data: null, text: "0"}); // high
                });
                hasDummySeries = true;
            }

            // add dummy data as category to queryResult
            if (titlesForChart.category.length === 0) {
                //builds series when the jaql has no dimension on category and only measures only on series
                $plotHighchartSeries.buildBoxPlotSeriesWithOnlyNumericOnSeries(titlesForChart, queryResult, series, $$setNget(xAxis, 'categories', []));
                notHaveColorsYet = false;
            } else {

                // builds series when the jaql has one dimension on Category and no on Value
                $plotHighchartSeries.buildBoxPlotChart(titlesForChart, queryResult.rows(), series, $$setNget(xAxis, 'categories'));


                // hide dummy series from legend
                if (hasDummySeries) {
                    series[0].enableMouseTracking = false;
                    series[0].showInLegend = false;
                }
            }

            // slice xAxis.categories to number of categories
            var categories = $$get(xAxis, 'categories'), catOrigLength;
            if (defined(categories) && categories.length > categoriesCapacity) {

                catOrigLength = categories.length;
                xAxis.categories = categories.slice(0, categoriesCapacity);

                // cut data from series belongs to the categories that have been removed
                for (var i = 0, l = series.length; i < l; i++) {
                    series[i].data = series[i].data.slice(0, categoriesCapacity);
                }

                if (defined(errors)) {
                    errors.push({
                        errorKey: default_chart_settings.exceededCategoriesCapacity,
                        args: [categoriesCapacity, catOrigLength]
                    });
                }
            }

            //sets outliers series
            if(enabledOutliers) {
                 var outliersData = addOutliers(outliers,categories, titlesForChart, titlesForChart.boxDividerColor);
                 series.push(outliersData);
            }

            // sets mask formatting to series if exist, otherwise null
            setBoxPlotSeriesMaskedFormat(series, $$get(titlesForChart, 'value'));

            // gets series and categories capacity from style, if not exist takes the default settings for them
            var seriesCapacity = default_chart_settings.seriesCapacity, categoriesCapacity = default_chart_settings.categoriesCapacity;

            if (defined(dataLimits)) {

                if (defined(dataLimits.seriesCapacity)) {
                    seriesCapacity = dataLimits.seriesCapacity;
                }

                if (defined(dataLimits.categoriesCapacity)) {
                    categoriesCapacity = dataLimits.categoriesCapacity;
                }
            }


            //set color from palette
            for (var i = 0, l = series.length; i < l; i++) {

                series[i].color =  titlesForChart.boxDividerColor;
            }
            //apply whisker styles only on the first series
            series[0].medianWidth = 1;
            series[0].maxPointWidth = 42;
            series[0].minPointWidth = 20;
            series[0].whiskerWidth = 1;
            series[0].whiskerLength = "100%";

            // ignore not numbers values in series
            ignoreNotNumbersInSeries(series, ['y']);

            return series;

        };

        // render by create new highcharts or update the existing one
        this.render = function (scope, lmnt, args) {

            // getting highcharts instance attached to the DOM
            var widget = scope.widget,
                element = scope.element,
                $lmnt = $(lmnt[0]),
                chart = lmnt.data('hc'),
                reason = args.reason,
                disableAnimations = window.disableAnimations || args.disableAnimations; // window.disableAnimations for Tests Automations

            var queryResult = $$.object.clone(widget.queryResult, true);

            // special case of long query and changing the layout before the refresh render, so the widgetredraw happened first where there is no chart at all
            if ($.isEmptyObject(queryResult)) {
                return;
            }

            // redraw the chart as it was last time for dragging the xAxis reason
            if (reason === 'axisDragged' && queryResult.chart) {
                queryResult.chart.drawOnlyAxes = true;
                queryResult.chart.axisDraggedInfo = args.axisDraggedInfo;
                queryResult.xAxis.lineColor = '#808080';
            }
            if (queryResult.chart) {
                queryResult.chart.drawOnlyAxes = (reason === 'axisDragged');
            }
            var maxCharactersPerLabelFromWidget = $$get(widget, 'options.maxCharsPerLabelXAxis');
            if (args.maxCharactersPerLabel || maxCharactersPerLabelFromWidget) {
                $$set(queryResult, 'xAxis.labels.maxCharactersPerLabel', (args.maxCharactersPerLabel || maxCharactersPerLabelFromWidget));
            }

            // add events to chart (need to be here cause switching between widget editor and the dashboard updating the widget and we have to reset the event)

            var chartEvents = $$setNget(queryResult, 'chart.events');

            // add slection events to the chart
            addSelectionEvents(widget, scope, queryResult, $lmnt);

            // scroller redraw event
            $.extend(chartEvents, setScrollerEvent(widget, {$lmnt: $lmnt, scope: scope}));

            // xAxis drag redraw event
            $.extend(chartEvents, this.setXAxisDragEvent(widget));

            // check if number of yAxis changed, if yes will have to redraw
            var redrawCartesianNumOfYxisChanged = $$get(chart, 'options.yAxis.length') !== $$get(queryResult, 'yAxis.length') && $$get(chart, 'options.$familyType') === 'cartesian';

            // re/creating chart
            if (!defined(chart) || !defined(chart.options) || redrawCartesianNumOfYxisChanged || reason === "widgetredraw" || reason === "widgetredrawseries" ||
                reason === "widgetredrawaddscrollbar" || reason === "axisDragged" || reason === "redrawAxisDragged") {

                // set static margins to chart plotBox when resize one of the axes
                $draggingAxis.setChartMarginsWhenResizeAxis(chart, $lmnt, queryResult, args);

                // resetting chart
                if (defined(chart) && defined(chart.options)) {

                    chart.destroy();
                    $lmnt.empty();
                    lmnt.data('hc', null);
                }

                if (reason === "widgetredraw") {
                    $$set(queryResult, 'plotOptions.series.animation', false);
                }

                switch (queryResult.$familyType) {

                    case 'cartesian':

                        var xAxis = $$get(queryResult, 'xAxis');
                        var resetMinMaxXAxis = (reason === 'widgetredrawseries' || (widget.options.dashboardFiltersMode === 'filter' && defined($$get(xAxis, 'min')) && defined($$get(xAxis, 'max'))));

                        // make the navigator to stay in place when selecting in highlight filter mode. #pris-9899
                        if (resetMinMaxXAxis && defined(args.resetScroller) && !args.resetScroller) {
                            resetMinMaxXAxis = args.resetScroller;
                        }

                        // sets AxisSetting of both axis according element size
                        setAxisAndScrollbarSettingDueToElementSize(widget, xAxis, $$get(queryResult, 'yAxis'), $$get(queryResult, 'navigator'), $$get(queryResult, 'series'), $$get(queryResult, 'plotOptions'), $$get(queryResult, 'chart.type'), $$get(queryResult, 'chart.polar'), $$get(widget, 'options.previousScrollerLocation'), resetMinMaxXAxis, $lmnt, element, scope.dashboard ? scope.dashboard.renderMode : false);
                        disableTitlesDueToElementSize($lmnt, queryResult);

                        // sets series show in legend and markers defaults due to element size
                        $highchartSeries.setSeriesSettingOfShowHideDueToElementSize(queryResult, $$get(queryResult, 'series'), $$get(queryResult, 'legend'), $lmnt, element);

                        break;

                    case 'plotwidget':

                        var xAxis = $$get(queryResult, 'xAxis');
                        var resetMinMaxXAxis = (reason === 'widgetredrawseries' || (widget.options.dashboardFiltersMode === 'filter' && defined($$get(xAxis, 'min')) && defined($$get(xAxis, 'max'))));

                        // make the navigator to stay in place when selecting in highlight filter mode. #pris-9899
                        if (resetMinMaxXAxis && defined(args.resetScroller) && !args.resetScroller) {
                            resetMinMaxXAxis = args.resetScroller;
                        }

                        // sets AxisSetting of both axis according element size
                        //  setAxisAndScrollbarSettingDueToElementSize(widget, xAxis, $$get(queryResult, 'yAxis'), null, $$get(queryResult, 'series'), $$get(queryResult, 'plotOptions'), $$get(queryResult, 'chart.type'), null , null, null, $lmnt, element, scope.dashboard ? scope.dashboard.renderMode : false);
                        //  disableTitlesDueToElementSize($lmnt, queryResult);

                        // sets series show in legend and markers defaults due to element size
                        $plotHighchartSeries.setSeriesSettingOfShowHideDueToElementSize(queryResult, $$get(queryResult, 'series'), $$get(queryResult, 'legend'), $lmnt, element);

                        break;

                    case 'pie':

                        // remove pie's labels when the container is too small
                        removePieSmallLabels($lmnt, $$get(queryResult, 'plotOptions.pie'));

                        break;

                    case 'polar':
                        disableTitlesDueToElementSize($lmnt, queryResult);
                        break;

                    case 'scatter':

                        // sets AxisSetting of both axis according element size
                        setAxisAndScrollbarSettingDueToElementSize(widget, $$get(queryResult, 'xAxis'), $$get(queryResult, 'yAxis'), null, $$get(queryResult, 'series'), null, $$get(queryResult, 'chart.type'), null, null, null, $lmnt, element);
                        disableTitlesDueToElementSize($lmnt, queryResult);
                        break;

                    case 'table':

                        break;

                    default:

                        break;
                }

                // returns to editor decision on navigator when exitting wizard new widget mode
                if (widget.appstate !== 'newwidget' && $$get(queryResult, 'legend.enabledFromEditor') !== $$get(queryResult, 'legend.enabled') && $$get(queryResult, 'legend.enabled') !== true) {
                    $$set(queryResult, 'legend.enabled', $$get(queryResult, 'legend.enabledFromEditor'));
                }

                // change legend setteings due to container width and height
                changeLegendSettingDueToElementSize($$get(queryResult, 'legend'), $lmnt);

                if ($$get(queryResult, 'initDummyChart') === true || $$get(queryResult, 'series.0.name') === 'tooManyPieSlices') {

                    var elementHeight = $lmnt.height();//element.height ? parseInt(element.height, 10) : $lmnt.height();

                    var marginVertical = elementHeight / 4, marginHorizonal = $lmnt.width() / 4;

                    $$set(queryResult, 'plotOptions.series.animation', false);
                    $$set(queryResult, 'chart.margin', [marginVertical, marginHorizonal, marginVertical, marginHorizonal]);
                }

                // remove all pie's style if the only slice is 'Others'
                if ($$get(queryResult, 'series.0.name') === 'tooManyPieSlices') {

                    $$set(queryResult, 'legend.enabled', false);
                    $$set(queryResult, 'plotOptions.pie.dataLabels.enabled', false);
                    $$set(queryResult, 'plotOptions.pie.dataLabels.showPercentLabels', false);
                    queryResult.plotOptions.pie.point.events.mouseOver = function () {
                        return false;
                    };
                }

                // disable chart animations
                setAnimationProperty(queryResult.series, disableAnimations);

                // allow override highcharts options
                var beforeViewLoadedArgs = {widget: widget, options: queryResult, element: $lmnt};
                widget.trigger('beforeviewloaded', beforeViewLoadedArgs);

                $lmnt.highcharts(queryResult);

                var chartDraw = $lmnt.highcharts();

                // draw errors panel box
                var chartErrors = $$get(chartDraw, 'options.errors');
                if (defined(chartErrors) && chartErrors.length > 0) {
                    drawErrorsBox(chartDraw, chartErrors);
                }

            }

            // updating existing highchart instance
            else {

                // allow override highcharts options - in the update section there is meaning to beforeviewoaded only for series data and axes manipulations
                var beforeViewLoadedArgs = {widget: widget, options: queryResult, element: $lmnt};
                widget.trigger('beforeviewloaded', beforeViewLoadedArgs);

                var chartType = $$get(chart, 'options.chart.type');
                var oldElement = [], newElement = [];

                if (defined($$get(chart, 'options.chart.margin'))) {

                    args.reason = "widgetredrawseries";
                    this.render(scope, lmnt, args);
                    return;
                }

                // gets series name and data
                switch (chartType) {
                    case 'pie':
                        oldElement = _.map($$get(chart, 'series'), function (ser) {
                            return {
                                name: ' ',
                                animation: ser.animation,
                                data: ser.points ? _.map(ser.points, function (point) {
                                    return [point.name, point.y];
                                }) : []
                            };
                        });
                        break;
                    default:

                        oldElement = _.map($$get(chart, 'series'), function (ser) {
                            return {
                                name: ser.name,
                                animation: ser.animation,
                                data: ser.data,
                                color: ser.color,
                                yAxis: $$get(ser, 'options.yAxis'),
                                visible: ser.visible
                            };
                        });
                        oldElement = _.filter(oldElement, function (ser) {
                            return ser.name !== 'Navigator';
                        });
                        break;
                }

                // reason for render is color change -> data is the same!
                if (angular.isObject(reason) && reason.reason === "colorChange") {

                    $highchartSeries.updateSeriesColor(chart, reason.member, reason.color);
                    return;
                }

                // get the new data of series
                newElement = _.map($$get(queryResult, 'series'), function (ser) {
                    var visible = ser.visible;
                    return {
                        name: ser.name,
                        animation: ser.animation,
                        data: ser.data,
                        color: ser.color,
                        visible: defined(visible) ? ser.visible : true,
                        yAxis: ser.yAxis,
                        type: ser.type,
                        showInLegend: ser.showInLegend,
                        mask: ser.mask
                    }
                });

                // reset scroller if the categories has been changed
                var resetScroller = $$get(chart, 'options.xAxis.' + 0 + '.categories.length') !== $$get(queryResult, 'xAxis.categories.length');

                // check for the possibility of updating the chart, create a new highcharts if failed
                if (!$plotHighchartSeries.checkForUpdateChart(chart, oldElement, newElement, queryResult.xAxis)) {

                    args.reason = "widgetredrawseries";
                    args.resetScroller = resetScroller;
                    this.render(scope, lmnt, args);
                    return;
                }

                // updates points of series
                $plotHighchartSeries.updateCartesianSeries(chart, oldElement, newElement, queryResult, $lmnt, element);

                if ($$get(queryResult, 'chart.type') !== "pie") {

                    if (resetScroller && ($$get(chart, 'options.navigator.enabled') !== $$get(queryResult, 'navigator.enabled'))) {

                        args.reason = "widgetredrawaddscrollbar";
                        this.render(scope, lmnt, args);
                        return;
                    }

                    // sets AxisSetting of both axis according element size
                    setAxisAndScrollbarSettingDueToElementSize(widget, $$get(queryResult, 'xAxis'), $$get(queryResult, 'yAxis'), $$get(queryResult, 'navigator'), $$get(queryResult, 'series'), $$get(queryResult, 'plotOptions'), $$get(queryResult, 'chart.type'), $$get(queryResult, 'chart.polar'), $$get(widget, 'options.previousScrollerLocation'), resetScroller, $lmnt, element);
                    disableTitlesDueToElementSize($lmnt, queryResult);
                }

                // check the need to redraw the chart to add the scrollbar
                if ($$get(queryResult, 'scrollbar.enabled') === true && !defined($$get(chart, 'options.scrollbar.enabled'))) {

                    args.reason = "widgetredrawaddscrollbar";
                    this.render(scope, lmnt, args);
                    return;
                }

                var labelsColorOriginal = $$get(chart, 'xAxis.' + 0 + '.options.labels.style.origColor');
                chart.xAxis[0].update($$get(queryResult, 'xAxis'), false);
                $$set(chart, 'xAxis.' + 0 + '.options.labels.style.origColor', labelsColorOriginal);

                for (var i = 0, l = $$get(queryResult, 'yAxis.length'); i < l; i++) {
                    chart.yAxis[i].update($$get(queryResult, 'yAxis.' + i), false);
                }


                // redrawing chart
                chart.redraw();

                // draw errors panel box
                var chartErrors = $$get(chart, 'options.errors');
                if (defined(chartErrors) && chartErrors.length > 0) {
                    drawErrorsBox(chart, chartErrors);
                }

            }

            // broadcast data to update the widget editor styler
            if (scope.$root.appstate === 'widget' && $$get(queryResult, 'chart.type') !== "pie") {
                $stylerUpdate.updateAxesRange(chartDraw || chart, widget.$noResults);
            }

            // attaching
            lmnt.data('hc', chartDraw);

            // attach dragging xAxis to control length of vertical labels if needed
            $draggingAxis.dragAxis($lmnt, chartDraw, $$get(chartDraw, 'xAxis.0'), reason, scope);

        };
    }]);