﻿//
// common highcharts wrapper service

mod.service('$plotHighchart.series', [
    'dashboard-base.services.$mask',
    '$filter',
    function ($mask, $filter) {

        // helper functions for building series and categories
        //
        // Boxplot Series

        // builds series when the jaql has no dimension on xAxis and only measures only on series
        this.buildBoxPlotSeriesWithOnlyNumericOnSeries = function (titlesForChart, queryResult, series, categories) {
            series.push({data: [], onlyMeasuresInSeries: true});

            var oneResult = queryResult.rows()[0];
            var title = $$get(titlesForChart, 'value.' + '0' + '.title');

            if (oneResult.length) {
                series[0].data.push({
                    x: 0,
                    low: oneResult[3].data,
                    q1: oneResult[0].data,
                    median: oneResult[1].data,
                    q3: oneResult[2].data,
                    high: oneResult[4].data,
                    color: $$get(titlesForChart, 'boxDividerColor') || '#c3c3c3',
                    fillColor: $$get(titlesForChart, 'boxFillColor') || '#c3c3c3',
                    innerBoxColor: $$get(titlesForChart, 'lowerQuartileColor') || '#c3c3c3',
                    selectionData: {0: oneResult[0].data},
                    queryResultIndex: 0
                });

                categories.push(title);
                series[0].name = title;
            } else {
                series.push({data: [], name: title, visible: false, showInLegend: true});
            }
        };

        //builds series when the jaql has one dimension on xAxis and measures only on series
        this.buildBoxPlotChart = function (titlesForChart, queryResult, series, categories) {

            _.each(titlesForChart.value, function (s) {
                series.push({name: s.title, data: [], yAxis: 0, type: s.singleSeriesType});
            });

            var i = 0, l = queryResult.length;
            var selected;
            var affectFromDashboardFilter = $$get(titlesForChart, 'category.0.affectFromGlobalFilter');

            for (; i < l; i++) {

                categories.push(queryResult[i][0].text);
                selected = affectFromDashboardFilter ? (queryResult[i][0].selected === true ? false : true) : false;

                series[0].data.push({
                    x: i,
                    low: queryResult[i][4].data,
                    q1: queryResult[i][1].data,
                    median: queryResult[i][2].data,
                    q3: queryResult[i][3].data,
                    high: queryResult[i][5].data,
                    color: $$get(titlesForChart, 'boxDividerColor') || '#c3c3c3',
                    fillColor: $$get(titlesForChart, 'boxFillColor') || '#c3c3c3',
                    innerBoxColor: $$get(titlesForChart, 'lowerQuartileColor') || '#c3c3c3',
                    selected: selected,
                    //  marker: { enabled: affectFromDashboardFilter ? true : null },
                    selectionData: {0: queryResult[i][0].data},
                    queryResultIndex: i
                });
            }

        };

        // helper functions for updating series
        // check if update or render a new highcharts
        this.checkForUpdateChart = function (chart, oldElement, newElement, newXAxis) {

            // call redraw if more than one series type
            if (_.union(_.pluck(newElement, 'type')).length !== 1) {
                return false;
            }

            // check if one of the old series has more points than the current to be draw.
            // reason - remove Points with highcharts is a very expensive method, and we will prefer to create new highcharts without the animation of update.
            for (var i = 0, l = oldElement.length; i < l; i++) {

                // case of remove series item
                if (!defined($$get(newElement, i + '.data'))) {
                    return false;
                }

                // case of remove points of series
                if (oldElement[i].data.length > newElement[i].data.length) {
                    return false;
                }

                // case of reorder measures with different yAxis index
                if (oldElement[i].yAxis !== newElement[i].yAxis) {
                    return false;
                }
            }

            var chartType = $$get(chart, 'options.chart.type');

            // first condition - validate that chartType is defined is indeed strange and it for case like the one described in pris-6748
            if (!defined(chartType) || chartType === 'bubble' || chartType === 'pie' || chartType === 'boxplot'
                || ($$get(chart, 'options.xAxis.0.plotBands') && oldElement[0].name != '$dummySeries') || ($$get(chart, 'options.xAxis.0.plotLines') && oldElement[0].name != '$dummySeries')
                || ($$get(chart, 'options.yAxis.0.plotBands') && oldElement[0].name != '$dummySeries') || ($$get(chart, 'options.yAxis.0.plotLines') && oldElement[0].name != '$dummySeries')
                || $$get(newXAxis, 'plotBands.length') > 0) {

                return false;
            }

            // check if there is a selection (must redraw)
            if ($$get(newElement, 0 + '.data.' + 0 + '.marker.enabled') !== $$get(oldElement, 0 + '.data.' + 0 + '.marker.enabled')) {
                return false;
            }

            return true;

        };

        // updates cartesian's series
        this.updateCartesianSeries = function (chart, oldElement, newElement, queryResult, $lmnt, element) {

            // indexes of real series, helper for ignoring the navigator series
            var indexesRealSeries = [], hasNavigator = false;
            _.each(chart.series, function (ser, index) {
                if (ser.name !== 'Navigator') {
                    indexesRealSeries.push(index);
                } else {
                    hasNavigator = true
                }
            });

            var currentSeries, currentPoint, i, l, j, m;

            // change the names of series and create new series
            for (i = 0, l = newElement.length; i < l; i++) {

                // check for change in series number #i - name property
                if ($$get(oldElement[indexesRealSeries[i]], 'name') !== $$get(newElement[i], 'name') || $$get(oldElement[indexesRealSeries[i]], 'color') !== $$get(newElement[i], 'color')
                    || $$get(oldElement[indexesRealSeries[i]], 'visible') !== $$get(newElement[i], 'visible') || $$get(oldElement[indexesRealSeries[i]], 'animation') !== $$get(newElement[i], 'animation')) {

                    currentSeries = $$get(chart.series[indexesRealSeries[i]], '');
                    if (!defined(currentSeries)) {

                        chart.addSeries({
                            name: newElement[i].name,
                            data: [],
                            mask: newElement[i].mask,
                            color: newElement[i].color,
                            visible: newElement[i].visible,
                            animation: newElement[i].animation
                        });
                        indexesRealSeries.push(($$get(newElement, i + '.visible') === false) ? 0 : (i + (hasNavigator ? 1 : 0)));
                    }

                    else {

                        if (currentSeries.name !== newElement[i].name) {

                            if (newElement[i].name !== "$dummySeries") {
                                currentSeries.update({
                                    name: newElement[i].name,
                                    color: newElement[i].color,
                                    showInLegend: true,
                                    enableMouseTracking: true,
                                    visible: newElement[i].visible,
                                    animation: newElement[i].animation
                                });
                            }
                            else {
                                currentSeries.update({
                                    name: newElement[i].name,
                                    color: newElement[i].color,
                                    showInLegend: false,
                                    enableMouseTracking: false,
                                    visible: newElement[i].visible,
                                    animation: newElement[i].animation
                                });
                            }
                        }

                        else {

                            currentSeries.update({
                                color: newElement[i].color,
                                visible: newElement[i].visible,
                                animation: newElement[i].animation
                            });
                        }
                    }
                }
            }

            // show / hide legend and markers according container size
            this.setSeriesSettingOfShowHideDueToElementSize(chart, newElement, $$get(queryResult, 'legend'), $lmnt, element);

            var yToInsert;

            // updates the data of series
            for (i = 0, l = newElement.length; i < l; i++) {

                currentSeries = $$get(chart, 'series.' + indexesRealSeries[i]);

                for (j = 0, m = newElement[i].data.length; j < m; j++) {

                    currentPoint = $$get(currentSeries, 'data.' + j);
                    if (!defined(currentPoint)) {

                        itemToInsert = newElement[i].data[j];

                        if (isNaN(itemToInsert)) {
                            currentSeries.addPoint({
                                y: $$get(itemToInsert, 'y'),
                                color: $$get(itemToInsert, 'color'),
                                marker: $$get(itemToInsert, 'marker'),
                                selected: $$get(itemToInsert, 'selected'),
                                selectionData: $$get(itemToInsert, 'selectionData')
                            }, false)
                        }
                        else {
                            currentSeries.addPoint({
                                y: itemToInsert,
                                marker: $$get(itemToInsert, 'marker'),
                                selected: $$get(itemToInsert, 'selected')
                            }, false);
                        }
                    }

                    else {

                        if (!defined(newElement[i].data[j].color)) {
                            newElement[i].data[j].color = currentSeries.color;
                        }
                        currentPoint.update(newElement[i].data[j], false);
                    }
                }

            }

        };

        // update series / points coloring
        this.updateSeriesColor = function (chart, member, color) {

            var familyType = $$get(chart, 'options.$familyType'),
                sItem;

            switch (familyType) {

                case 'cartesian':
                case 'polar':
                case 'scatter':
                case 'plotwidget':
                    sItem = _.find(chart.series, function (si) {
                        return si.name === member
                    });
                    break;

                case 'pie':
                    sItem = _.find(chart.series[0].points, function (p) {
                        return p.name === member;
                    });
                    break;
            }

            if (sItem) {

                if (defined($$get(sItem, 'points.0.options.color'))) {
                    for (var i = 0, l = sItem.points.length; i < l; i++) {
                        sItem.points[i].update({color: color}, false);
                    }

                    if (familyType === 'scatter') {
                        sItem.update({color: color}, false);
                    }

                    // redrawing chart
                    chart.redraw();
                }

                else {
                    sItem.update({color: color});

                    if (sItem.connector) {
                        sItem.connector.attr({stroke: color});
                    }
                }

            }
        };

        // sets series show in legend and markers defaults due to element size
        this.setSeriesSettingOfShowHideDueToElementSize = function (chart, series, $$legend, $lmnt, element) {

            // element validation
            if (!$lmnt || series.length === 0) {
                return;
            }

            var elementWidth = $lmnt.width(),
                elementHeight = $lmnt.height();//element.height ? parseInt(element.height, 10) : $lmnt.height();

            var legendHorizonalAlign, legendVerticalAlign;

            if (defined($$legend)) {

                legendHorizonalAlign = $$get($$legend, 'align'), legendVerticalAlign = $$get($$legend, 'verticalAlign');
            }

            var willHideLegend = (elementWidth < 260 && legendHorizonalAlign != 'center') || (elementHeight < 140 && legendVerticalAlign != 'middle') || series[series.length - 1].name === '$dummySeries';

            var chartSeries = $$get(chart, 'series'),
                hideInLegend;

            if (chartSeries) {

                var cleanSeriesIndex = 0; // the i (next line) is include the navigator

                for (var i = 0, l = chartSeries.length; i < l; i++) {

                    hideInLegend = willHideLegend || chartSeries[i].name === 'Navigator' || series[cleanSeriesIndex].showInLegend === false;

                    if (chartSeries[i].name !== 'Navigator') {
                        cleanSeriesIndex++;
                    }

                    if (chartSeries[i].update) {
                        chartSeries[i].update({
                            showInLegend: !hideInLegend
                        });
                    }
                    else {
                        chartSeries[i].showInLegend = !hideInLegend;
                    }

                }

            }

        };

    }]);