﻿//
// common highcharts wrapper service - tooltip

mod.service('$plotHighchart.tooltip', [
    'ux-controls.services.$dom',
    '$filter',
    function ($dom, $filter) {

        // number masking
        var numberMask = $filter('number');

        // helper functions for generate  the tooltips according the type and series

        var generateTooltipWithTemplate = function (widget, titlesForChart, queryResult, familyType, template, settings) {

            var tipper,
                tipscope = {pointScope: {}, pointMetadata: {}},
                titlesForChartFlatten = _.flatten(titlesForChart);

            return function (ev) {
                switch (familyType) {

                    case 'plotwidget':

                        var mask = $$get(this, 'series.options.mask');
                        var isMask = defined(mask);

                        tipscope.memberColor = this.color || this.series.color;
                        // content
                        tipscope.whiskersHeader = "Whiskers";
                        tipscope.boxHeader = "Box";

                        tipscope.whiskersMax = isMask ? mask(this.high) : this.high;
                        tipscope.whiskersMin = isMask ? mask(this.low) : this.low;
                        tipscope.boxMin = isMask ? mask(this.q1) : this.q1;
                        tipscope.boxMax = isMask ? mask(this.q3) : this.q3;
                        tipscope.boxMedian = isMask ? mask(this.median) : this.median;
                        tipscope.yOutliers = isMask ? mask(this.y) : this.y;
                        tipscope.nameOutliers = $$get(this, 'series.name');
                        tipscope.category = ($$get(this, 'series.xAxis.categories.length') > 0) ? this.series.xAxis.categories[this.x] : (this.xMask ? this.xMask(this.x) : numberMask(this.x));

                        // position
                        tipscope.x = $$get(ev, 'currentTarget.tooltipPos.0') || $$get(document, 'mousePosition.x') || $$get(event, 'clientX');
                        tipscope.y = $$get(ev, 'currentTarget.tooltipPos.1') || $$get(document, 'mousePosition.y') || $$get(event, 'clientY');
                        break;
                }

                // update the hover point for custom template
                $.extend(tipscope.pointScope, this);

                // update the hover point with the point metadata
                tipscope.pointMetadata = getPointMetadata(widget, queryResult, titlesForChartFlatten, this);

                // allow override tooltip default behavior
                var cancelTooltip = false,
                    beforedatapointtooltipArgs = {
                        widget: widget,
                        context: tipscope,
                        template: template,
                        cancel: cancelTooltip
                    };

                widget.trigger('beforedatapointtooltip', beforedatapointtooltipArgs);

                // set tipper for the first time
                if (!tipper) {
                    tipper = $dom.tip({
                        scope: tipscope,
                        template: beforedatapointtooltipArgs.template
                    }, {
                        css: 'charts',
                        placement: {
                            place: 'b',
                            anchor: 'l'
                        },
                        initialShowDelay: 0,
                        betweenShowDelay: 0
                    });
                }

                if (!beforedatapointtooltipArgs.cancel) {
                    tipper.activate({x: tipscope.x, y: tipscope.y, space: 0});
                    this.series.chart.sisenseTipper = {
                        familyType: familyType,
                        type: this.series.type,
                        tipper: tipper,
                        status: 'on'
                    };
                }
            };
        };

        var generateBoxPlotTooltip = function () {

            return '<div class="boxplot-tooltip">' +

                '<div class="boxplot-tooltip-whiskers">' +
                '<div class="boxplot-tooltip-header"> {{model.whiskersHeader}}  </div>' +

                '<div class="boxplot-tooltip-data">' +
                '<div class="boxplot-tooltip-value-name">Max</div>' +
                '<div class="spacer"></div>' +
                '<div class="boxplot-tooltip-value" data-ng-style="{color:model.memberColor}" > {{model.whiskersMax}}   </div>' +
                '</div>' +

                '<div class="boxplot-tooltip-data">' +
                '<div class="boxplot-tooltip-value-name" >Min</div>' +
                '<div class="boxplot-tooltip-value" data-ng-style="{color:model.memberColor}"> {{model.whiskersMin}}   </div>' +
                '</div>' +

                '<div class="highcharts-tooltip-line-separator"></div>' + // line separator
                '</div>' +


                '<div class="boxplot-tooltip-box">' +
                '<div class="boxplot-tooltip-header"> {{model.boxHeader}} </div>' +

                '<div class="boxplot-tooltip-data">' +
                '<div class="boxplot-tooltip-value-name" > Max  </div>' +
                '<div class="boxplot-tooltip-value" data-ng-style="{color:model.memberColor} "> {{model.boxMax}}   </div>' +
                '</div>' +

                '<div class="boxplot-tooltip-data">' +
                '<div class="boxplot-tooltip-value-name" "> Min  </div>' +
                '<div class="boxplot-tooltip-value" data-ng-style="{color:model.memberColor}"> {{model.boxMin}}   </div>' +
                '</div>' +

                '<div class="boxplot-tooltip-data">' +
                '<div class="boxplot-tooltip-value-name" "> Median  </div>' +
                '<div class="boxplot-tooltip-value" data-ng-style="{color:model.memberColor}"> {{model.boxMedian}}   </div>' +
                '</div>' +

                '<div class="highcharts-tooltip-line-separator"></div>' + // line separator

                '</div>' +

                '<div class="boxplot-tooltip-category">' +
                '<div class="boxplot-tooltip-value-category"> {{model.category}} </div>' +
                '</div>' +

                '</div>';
        };

        var getPointMetadata = function getPointMetadata(widget, queryResult, titlesForChartFlatten, point) {

            var pointMetadata = [],
                queryResultMetadata = queryResult.metadata(),
                queryResultIndex = point.queryResultIndex;

            // validate quertResult index
            if (!defined(queryResultIndex) || isNaN(queryResultIndex)) {
                return pointMetadata;
            }

            var valuesPanel = widget.metadata.panel('values'),
                valuesItems = valuesPanel ? valuesPanel.getEnabledItems() : null,
                numOfValuesItems = $$get(valuesItems, 'length');

            _.each(titlesForChartFlatten, function (item) {

                if (!defined(item.indexInJaqlMetadata) || (item.isMeasure && numOfValuesItems > 1 && item.title !== point.series.name)) {
                    return;
                }

                pointMetadata.push({
                    jaql: queryResultMetadata[item.indexInJaqlMetadata].jaql,
                    member: $$get(queryResult.$$rows, queryResultIndex + '.' + (item.indexInJaqlMetadata) + '.data')
                });
            });

            return pointMetadata;
        };


        // sets the proper tooltip only for boxplot
        this.generateTooltip = function (widget, titlesForChart, queryResult, familyType, settings, s, event) {

            var template;

            switch (familyType) {

                case 'plotwidget':
                    template = generateBoxPlotTooltip();
                    break;

                default:
                    break;
            }

            return generateTooltipWithTemplate(widget, titlesForChart, queryResult, familyType, template, settings, event);

        };


        // sets the proper tooltip for outliers
        this.generateTooltipOutliers = function (widget, titlesForChart, queryResult, familyType, settings, s, event) {


            var template =
                '<div class="cartesian-tooltip">' +
                '<div class="cartesian-tooltip-values">' +
                '<div class="cartesian-tooltip-seriesName"> {{model.nameOutliers}} </div> ' +
                '<div class="cartesian-tooltip-singleLineValue" data-ng-style="{color:model.memberColor}">' +
                '<div class="cartesian-tooltip-value"> {{model.yOutliers}} </div> ' +
                '</div>' +
                '</div>' +
                '<div class="highcharts-tooltip-line-separator"></div>' +
                '<div class="cartesian-tooltip-category">{{model.category}} </div> </div>' +
                '</div>';

            return generateTooltipWithTemplate(widget, titlesForChart, queryResult, familyType, template, settings, event);
        };

    }]);